/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_dac.h"             */
/*  Constants and definitions for DAC.                                       */
/*                                                                           */
/*  Author : AL/JYA                     Last modification : 08/08/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  High level DAC driver definitions - Dac_drvdef
            2.2  Low level DAC driver interface - Dac_desc
            2.3  DAC device context - Dac_ctx
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef DRV_DAC_H                      /* If this file is not already        */
#define DRV_DAC_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Registers ................................................................*/

#define DAC_NBREG                 9    /* Number of registers                */

/* Flags for supported bit depth per sample .................................*/

#define DAC_8BIT          (1U <<  8)   /* Device support 8 bit samples.      */
#define DAC_12BIT         (1U << 12)   /* Device support 12 bit samples.     */
#define DAC_16BIT         (1U << 16)   /* Device support 16 bit samples.     */
#define DAC_20BIT         (1U << 20)   /* Device support 20 bit samples.     */
#define DAC_24BIT         (1U << 24)   /* Device support 24 bit samples.     */
#define DAC_28BIT         (1U << 28)   /* Device support 28 bit samples.     */
#define DAC_32BIT         (1U <<  0)   /* Device support 32 bit samples.     */

/* Size of static buffers ...................................................*/

#define SMPBUF_SZ              1024    /* 1024 bytes per sample buffer.      */

/*
    2    Structures and typedefs
    ____________________________
*/

/*
   2.1  High level DAC driver definitions - Dac_drvdef
   ---------------------------------------------------
*/


typedef struct                         /* DAC driver definitions.            */
  BEGIN                                /* ---------------------------------- */
    UINT           lctx              ; /* Get start-up information on device */
    INT            nbch              ; /* Number of channels supported.      */
    UINT           supp_depth        ; /* Sample depth supported by device.  */
    UINT           supp_dma          ; /* Sample depth supported thru DMA.   */
    UINT           vref              ; /* Max voltage output (dV). (<15.0V)  */
  END              Dac_drvdef        ; /* ---------------------------------- */


/*
   2.2  Low level DAC driver interface - Dac_devdesc
   --------------------------------------------------
*/

typedef struct                         /* DAC interface                      */
  BEGIN                                /* ---------------------------------- */
    void         (*info         )()  ; /* Get start-up information on device */
    void         (*setfmt       )()  ; /* Configure format of data.          */
    void         (*itsmp        )()  ; /* Enable or disable interrupts.      */
    int          (*putsmp       )()  ; /* Give a new sample.                 */
    int          (*snddma       )()  ; /* Give a buffer to convert via DMA.  */
  END              Dac_desc          ; /* ---------------------------------- */




/*
   2.3  DAC device context - Dac_ctx
   ---------------------------------
*/

typedef struct                         /* DAC device context                 */
  BEGIN                                /* ---------------------------------- */
    Req_ctx        rctx              ; /* Allocation context.   req_lib.c    */
    Key_lnk        key               ; /* Intern job queues.    req_lib.c    */
    IWORD          lctx              ; /* Low-level device/context index     */
                                       /*                                    */
    Dac_drvdef    *def               ; /* Device generic definitions.        */
    void         (*setfmt       )()  ; /* Configure format of data.          */
    void         (*itsmp        )()  ; /* Enable or disable interrupts.      */
    int          (*putsmp       )()  ; /* Give a new sample.                 */
    int          (*snddma       )()  ; /* Give a batch of samples.           */
                                       /*                                    */
    UINT           reg  [DAC_NBREG]  ; /* Registers                          */
                                       /*                                    */
    IWORD          dma               ; /* DMA in use (1) or not (0).         */
                                       /*                                    */
                                       /* Input format description --------- */
    IWORD          isz               ; /* Size in bytes of a sample.         */
    IWORD          ipitch            ; /* Distance between samples of a cnl. */
    UCHAR          iorder       [4]  ; /* Byte reordering.                   */
                                       /*                                    */
                                       /* Output format description -------- */
    IWORD          osz               ; /* Size in bytes of a sample.         */
    IWORD          opitch            ; /* Distance between samples of a cnl. */
    IWORD          omsk              ; /* Mask to apply to sample.           */
                                       /*                                    */
                                       /* Format conversion ---------------- */
    IWORD          noconv            ; /* Flag to avoid conversion altogether*/
    IWORD          sof7              ; /* Offset (shift) to apply to sample. */
                                       /*                                    */
                                       /* Input data coordinates ----------- */
    UCHAR         *psnd              ; /* Pointer on data to send.           */
    IWORD          nsnd              ; /* Number of remaining bytes to send. */
                                       /*                                    */
                                       /* Conversion buffers --------------- */
    UCHAR          buf          [2]    /* Expansion buffers for samples.     */
                        [SMPBUF_SZ]  ; /*                                    */
    IWORD          nbuf         [2]  ; /* Number of samples in "buf[i]".     */
    UINT          *smp0         [2]  ; /* Left channel pointer in "buf[i]".  */
    UINT          *smp1         [2]  ; /* Right channel pointer in "buf[i]". */
    IWORD          wbuf              ; /* Writing index in "buf".            */
    IWORD          rbuf              ; /* Reading index in "buf".            */
                                       /*                                    */
    UINT          *smp          [2]  ; /* Samples for both channels.         */
    IWORD          nsmp              ; /* Number of samples per channel.     */
    IWORD          ismp              ; /* Next sample to send.               */
                                       /* For "clean" driver close --------- */
    IWORD          writenb           ; /* Number of non completed writes     */
    S_evt          req_close         ; /* Copy of the REQ_CLOSE event        */
                                       /* Debug counters                     */
    IWORD          cpt_write         ;
    IWORD          cpt_osnd          ;
    IWORD          cpt_evtend        ;
    IWORD          cpt_dacevtend     ;
    IWORD          cpt_itsmp         ;
    IWORD          cpt_itdma         ;
    IWORD          cpt_ittxsmp       ;
    IWORD          cpt_ittxdma       ;
    IWORD          val[10]           ;
    IWORD          valpt             ;

  END              Dac_ctx           ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern UCHAR            dac_numdrv   ; /* Driver identifier for board access.*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/

void  itdacsmp                         (IWORD                                );
void  itdacdma                         (IWORD                                );


#endif                                 /* End of DRV_DAC_H.                  */
