/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_dma.h"             */
/*  Constants and definitions for DMA driver.                                */
/*                                                                           */
/*  Author : RG/FA                      Last modification : 02/11/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  Board structure for low-level procedure addresses
            2.2  Board structure for high level driver parameters
            2.3  Source or destination parameter
            2.4  One DMA request description
            2.5  One DMA channel context

         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DRV_DMA_H                      /* If this file is not already        */
#define DRV_DMA_H                      /* included, include it.              */

#ifdef NDRV_DMA_C

/*
    1    Defines
    ____________
*/

/* Values for the "code" field od Dma_job ...................................*/

#define O_MEMSND             0         /* copy from RAM to register          */
#define O_MEMRCV             1         /* copy from register to RAM          */
#define O_MEMCPY             2         /* use the memcpy function            */
#define O_DMASND             3         /* use the "dma_syx_snd" function     */
#define O_DMARCV             4         /* use the "dma_syx_rcv" function     */
#define O_DMACPY             5         /* use the "dma_syx_cpy" function     */

/* Values for the "opt" field of Dma_job ....................................*/

#define O_SWAP2          0x0001        /* Two  bytes swap supported          */
#define O_SWAP4          0x0002        /* Four bytes swap supported          */

#define O_SRCWRP         0x0004        /* Automatic source addr wrap         */
#define O_DSTWRP         0x0008        /* Automatic destination addr wrap    */
#define O_SRCSZ1         0x0100        /* Source is a 1 byte wide register   */
#define O_SRCSZ2         0x0200        /* Source is a 2 bytes wide register  */
#define O_SRCSZ4         0x0400        /* Source is a 4 bytes wide register  */
#define O_DSTSZ1         0x1000        /* Dest.  is a 1 byte wide register   */
#define O_DSTSZ2         0x2000        /* Dest.  is a 2 bytes wide register  */
#define O_DSTSZ4         0x4000        /* Dest.  is a 4 bytes wide register  */

/* Values for the "type" field of S_mem .....................................*/

#define REG1                  1        /* Area is a 1 byte size register     */
#define REG2                  2        /* Area is a 2 byte size register     */
#define REG4                  4        /* Area is a 4 byte size register     */
#define SEG                   5        /* Area is a RAM segment              */
#define CIR                   6        /* Area is a RAM circular buffer      */

/*
    2    Structures and typedefs
    ____________________________
*/

/* 2.1  Board structure for low-level procedure addresses ...................*/

typedef struct                         /* DMA Low level procedures           */
  BEGIN                                /* ---------------------------------- */
    IWORD             (*info)    ()  ; /* Name of the DMA Device             */
    IWORD             (*init)    ()  ; /* Initializes the DMA controller     */
    IWORD             (*end)     ()  ; /* End of use of the DMA controller   */
    IWORD             (*init_hch)()  ; /* Initializes one hardware channel   */
    IWORD             (*end_hch) ()  ; /* End of use of one hardware channel */
    IWORD             (*snd)     ()  ; /* RAM ro register transfer start     */
    IWORD             (*rcv)     ()  ; /* Register to RAM transfer start     */
    IWORD             (*cpy)     ()  ; /* RAM to RAM transfer start          */
    IWORD             (*abort)   ()  ; /* Aborts an on-going transfer        */
  END                 Dma_desc       ; /* ---------------------------------- */


/* 2.2  Board structure for high level driver parameters ....................*/

typedef struct                         /* DMA High level driver params       */
  BEGIN                                /* ---------------------------------- */
    UINT              ldevnum        ; /* Device index for low level driver  */
    UCHAR             nbchan         ; /* Number of channels                 */
    UCHAR             type           ; /* 0:General Purpose 1:Specialized    */
    const UCHAR       *maxuser       ; /* Maximum numbers of allowed alloc's */
    USHORT            opt            ; /* Available options bitmaps          */
    USHORT            thr            ; /* Memcpy/DMA usage limit             */
  END                 Dma_drvdef     ; /* ---------------------------------- */


/* 2.3  Source or destination parameter .....................................*/

typedef struct                         /* Source or destination DMA area     */
  BEGIN                                /* ---------------------------------- */
    UCHAR             *adr           ; /* Area base address                  */
    UINT              size           ; /* Area size in bytes                 */
    UCHAR             type           ; /* Area description bits              */
                                       /* ---------------------------------- */
    UINT              inoff          ; /* Next byte to write offset          */
    UINT              incnt          ; /* Bytes beeing inputted              */
    UINT              outoff         ; /* Next byte to read offset           */
    UINT              outcnt         ; /* Bytes beeing outputted             */
    UINT              cnt            ; /* Byte count                         */
  END                 S_mem          ; /* ---------------------------------- */


/* 2.4  One DMA request description .........................................*/

typedef struct        dma_job          /* One DMA request                    */
  BEGIN                                /* ---------------------------------- */
    UCHAR             code           ; /* Code of the requested operation    */
    UCHAR             *src           ; /* Transfer data source               */
    UCHAR             *dst           ; /* Transfer data destination          */
    UCHAR             srctype        ; /* Source description type            */
    UCHAR             dsttype        ; /* Desctination description type      */
    UINT              cnt            ; /* Number of bytes to be copied       */
    UINT              to             ; /* Time-out value in milliseconds     */
    USHORT            opt            ; /* Option bits for transfer           */
                                       /* ---------------------------------- */
    void              (*cb)()        ; /* End of transfer call-back addr     */
    void              *cbadr         ; /* First parameter to be given to cb  */
    UINT              userid         ; /* Channel identifier                 */
    INT               reqid          ; /* 2nd   parameter to be given to cb  */
    S_mem            *srcmem         ; /* Source area structure address      */
    S_mem            *dstmem         ; /* Destination area structure address */
    struct dma_job   *next           ; /* Link addr to next job              */
  END                 Dma_job        ; /* ---------------------------------- */
 

/* 2.5  One DMA channel context .............................................*/

typedef struct                         /* One DMA channel context            */
  BEGIN                                /* ---------------------------------- */
    UCHAR             dev            ; /* DMA controller high level number   */
    UCHAR             hchan          ; /* DMA channel    low level  number   */
    UCHAR             usernb         ; /* 0:free >0:allocated                */
    UCHAR             wdma           ; /* 1: Waiting for a DMA interrupt Re  */
    UINT              userbtm        ; /* Allocation bitmap                  */
    Dma_job          *job            ; /* Linked list of jobs head           */
    const Dma_desc   *desc           ; /* Low level procedures               */
    const Dma_drvdef *def            ; /* High level driver parameters       */
  END                 Dma_ctx        ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/



/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD  dma_init          (INT                                                );
IWORD  dma_end           (INT                                                );
IWORD  dma_alloc_gpchan  (UINT,INT*                                          );
IWORD  dma_alloc_spchan  (INT,INT,INT*                                       );
IWORD  dma_free_chan     (INT                                                );
IWORD  dma_start         (INT,S_mem*,S_mem*,INT,INT,UINT,UINT,
                          void(*)(),void*,INT                                );
IWORD  dma_abort         (INT, INT                                           );
IWORD  dma_io            (INT, Dma_job*, INT                                 );
IWORD  dma_cb_evt        (S_evt*,INT,INT,INT                                 );

IWORD  smem_init         (S_mem*,UCHAR*,UINT,INT                             );
void   smem_reset        (S_mem*                                             );
void   smem_start_input  (S_mem*,UINT                                        );
void   smem_end_input    (S_mem*,UINT                                        );
void   smem_start_output (S_mem*,UINT                                        );
void   smem_end_output   (S_mem*,UINT                                        );
UCHAR* smem_rdadr        (S_mem*                                             );
UCHAR* smem_wradr        (S_mem*                                             );
UINT   smem_free         (S_mem*                                             );
UINT   smem_avail        (S_mem*                                             );

void   itdmasnd          (INT,INT,INT                                        );
void   itdmarcv          (INT,INT,INT                                        );
void   itdmacpy          (INT,INT,INT                                        );
#else

/*
    1    Defines
    ____________
*/

/*  Definition du champ "type" des jobs DMA .................................*/

#define SRC_POINT        0x0001        /* La source      est un point        */
#define SRC_SEGMENT      0x0002        /* La source      est un segment      */
#define SRC_RECTANGLE    0x0004        /* La source      est un rectangle    */
#define SRC_LINK         0x0008        /* La source      est un link         */
#define SRC_AREA         0x0010        /* La source      est de type area    */
#define DST_POINT        0x0100        /* La destination est un point        */
#define DST_SEGMENT      0x0200        /* La destination est un segment      */
#define DST_RECTANGLE    0x0400        /* La destination est un rectangle    */
#define DST_LINK         0x0800        /* La destination est un link         */

/*  Definition du champs "hfunc" des jobs DMA ...............................*/

#define DMA_FUNC_SWAP2   0x0001        /* Le DMA gere les swap sur 2 octets. */
#define DMA_FUNC_SWAP4   0x0002        /* Le DMA gere les swap sur 4 octets. */
#define DMA_FUNC_LISTS   0x0004        /* Le DMA gere les listes de jobs.    */
#define DMA_FUNC_SLISTS  0x0008        /* Reserve pour drv_dma.c!            */
#define DMA_FUNC_TOGGLE  0x0010        /* Le DMA gere 2 jobs en parallele.   */
#define DMA_FUNC_PIXELS  0x0020        /* Le DMA copie des pixels.           */
#define DMA_FUNC_AREA    0x0030        /* Le DMA sait tracer les zones AREA. */
#define DMA_FUNC_GMSK    0x00FF        /* Masque les bits generiques.        */
#define DMA_FUNC_SMSK    0xFF00        /* Masque les bits specifiques.       */


/*
    2    Structures and typedefs
    ____________________________
*/


/*  Structure de description des devices DMA ................................*/

typedef struct                         /* Structure decrivant un device DMA  */
  BEGIN                                /* ---------------------------------- */
    void            (*init  )()      ; /* Procedure d'initialisation.        */
    void            (*end   )()      ; /* Procedure de finalisation.         */
    void            (*dma   )()      ; /* Procedure de transfert.            */
    UINT           *quanta           ; /* Quantum par defaut des canaux.     */
    USHORT          hfunc            ; /* Fonctions hardwares disponibles.   */
    UCHAR           nbhcnl           ; /* Nombre de hardware canaux du device*/
    UCHAR           mode             ; /* Mode d'utilisation du device.      */
    IWORD           maxlist          ; /* Taille max des listes de requetes. */
  END               Dma_desc         ; /* ---------------------------------- */


/* Structure decrivant un requete DMA ("job") ...............................*/

typedef struct Dma_job_s               /* Job DMA                            */
  BEGIN                                /* ---------------------------------- */

/* Parametres reserves a drv_dma.c ..........................................*/

    UCHAR           cnl              ; /* Numero de canal (logique).         */
    UCHAR           prio             ; /* Priorite du transfert              */
    USHORT          type             ; /* Type du transfert.                 */
                                       /*                                    */
    void           (*cbk )()         ; /* Procedure de callback              */
    UINT            p1               ; /* 1er parametre pour la callback     */
    UINT            p2               ; /* 2nd parametre pour la callback     */
                                       /*                                    */
    UINT            hlg              ; /* Nb d'octets a transfer.            */
    UINT            htodo            ; /* Nb d'octets restants a transferer. */
                                       /*                                    */
    UCHAR          *hasrc            ; /* Coordonnees de la source pour le   */
    UINT            hwsrc            ; /* haut niveau (ne tiennent pas compte*/
    UINT            hhsrc            ; /* du quantum).                       */
    UINT            hpsrc            ; /*                                    */
                                       /*                                    */
    UCHAR          *hadst            ; /* Coordonnees de la destination pour */
    UINT            hwdst            ; /* le haut niveau (ne tiennent pas    */
    UINT            hhdst            ; /* compte du quantum).                */
    UINT            hpdst            ; /*                                    */
                                       /*                                    */
    UINT            hline            ; /* Numero de Ligne courante           */
    UINT            hoff             ; /* Offset courant ligne courante      */
                                       /*                                    */
    struct          Dma_job_s *next  ; /* Job suivant de la liste            */
                   
/* Parametres utilisables par la couche basse ...............................*/
                   
    void           (*step)()         ; /* Procedure de fin de transfert      */
                                       /*                                    */
    UINT            todo             ; /* Nb d'octets a transferer.          */
    USHORT          hfunc            ; /* Fonctions hardware utilisees.      */
                                       /*                                    */
    UCHAR           fmt              ; /* Format des pixels.                 */
    UCHAR           alpha            ; /* Facteur de melange.                */
    UCHAR           color[4]         ; /* Couleur des pixels.                */
                                       /*                                    */
    UCHAR          *list             ; /* Ad buffer link (lnl2pnt, pnt2lnk). */
                                       /*                                    */
    UCHAR          *asrc             ; /* Coordonnees de la source pour la   */
    UINT            wsrc             ; /* couche basse (tiennent compte du   */
    UINT            hsrc             ; /* quantum).                          */
    UINT            psrc             ; /*                                    */
                                       /*                                    */
    UCHAR          *adst             ; /* Coordonnees de la destination pour */
    UINT            wdst             ; /* la couche basse (tiennent compte   */
    UINT            hdst             ; /* du quantum).                       */
    UINT            pdst             ; /*                                    */
  END               Dma_job          ; /* ---------------------------------- */


/*  Structure de description des canaux DMA .................................*/

typedef struct                         /* Contexte d'un canal DMA.           */
  BEGIN                                /* ---------------------------------- */
    UCHAR           dev              ; /* Numero du device associe au canal. */
    UCHAR           hcnl             ; /* Numero de canal hardware du device.*/
    UCHAR           nbu              ; /* Nombre d'utilisateurs references.  */
    UINT            quantum          ; /* Quantum actuel pour le canal.      */
    UINT            thr              ; /* Seuil memcpy / copie DMA.          */
    Dma_job        *ljob             ; /* Pointe sur liste des jobs du canal.*/
  END               Dma_cnl          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/

#endif                                 /* else NDRV_DMA_C                    */

#endif                                 /* End of DRV_DMA_H.                  */
