/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_led.h"             */
/*  Constants and definitions for LED driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 04/22/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
            1.1  High level interface - LED driver registers numbers
            1.2  High level interface - STATE tag values
            1.3  High level interface - COLOR tag values
            1.4  High level interface - CMD tag values

         2  Structures and typedefs
            2.1  Hardware dependant set of LED driver procedures - Led_desc
            2.2  LED controller context structure - Ledctrl_ctx
            2.3  One LED context structure - Led_ctx
            2.4  String byte store context - Ledbs_ctx

         3  Public variable
         4  Prototypes of public procedures
         5  RAM data to be inclueded in board_xyz.c


*/


#ifndef DRV_LED_H                      /* If this file is not already        */
#define DRV_LED_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    1.1  High level interface - LED driver registers numbers
    --------------------------------------------------------
*/



#define SZPATT              128        /* Size in bytes of PATTERN (text)    */
                                       /* Must be a multiple of 4            */

/* Needed registers for tags TYPE, NAME and PATTERN/RCUPATT .................*/

#define NBRTYPE               1        /* TYPE     1 UINTS    4 bytes        */
#define NBRNAME               2        /* NAME     2 UINTS    8 bytes        */
#define NBRPATT     (SZPATT/ZI)        /* PATTERN 32 UINTS  128 bytes        */

/* Order numbers of LED controller's associated registers ...................*/

#define NRTYPE                0        /* string  TYPE     (1  UINTs)        */
#define NRNAME                1        /* string  NAME     (2  UINTs)        */
#define NRNBLEDS              3        /* integer NBLED    (1  UINT)         */
#define NRDEVNUM              4        /* integer DEVNUM   (1  UINT)         */

/* Order numbers of LED associated registers ................................*/

#define NRLEDNUM              5        /* integer LEDNUM   (1  UINT)         */
#define NRLEDNAME             6        /* integer LEDNAME  (2  UINT)         */
#define NRCOLOR               8        /* integer COLOR    (1  UINT)         */
#define NRSTATE               9        /* integer STATE    (1  UINT)         */
#define NRVLRCU              10        /* integer VLRCU    (1  UINT)         */
#define NRPSTACK             11        /* integer PSTACK   (1  UINT)         */
#define NRLEDCMD             12        /* integer CMD      (1  UINT)         */
#define NRPATTERN            13        /* string PATTERN  (32 UINTs)         */
#define NRRCUPATT            45        /* string RCUPATT  (32 UINTs)         */

/* Tailles en UINTs des registres controlleurs et leds ......................*/

#define NBREGCTR              5        /* Taille du tableau de registres     */
                                       /* Registres  0 a 4 inclus            */
#define NBREGLED              5        /* Taille du tableau de registres     */
                                       /* Registres  5 a 9 inclus            */

/*
   1.2  High level interface - STATE tag values
   --------------------------------------------

   On notera que ces deu constantes peuvent aussi servir de parametrre d'entree
   a la procedure de bas niveau "setLED_xyz" (parametre state)
*/

#define ST_OFF                0        /* Led eteinte                        */
#define ST_ON                 1        /* Led allumee                        */

/*
   1.3  High level interface - COLOR tag values
   --------------------------------------------
*/
#define C_GREEN               0        /* Couleur verte                      */
#define C_YELLOW              1        /* Couleur jaune                      */
#define C_RED                 2        /* Couleur rouge                      */
#define C_BLUE                3        /* Couleur bleue                      */

/*
   1.4  High level interface - CMD tag values
   ------------------------------------------
*/
#define CMD_PUSH              0        /* Commande PUSH                      */
#define CMD_POP               1        /* Commande POP                       */
#define CMD_POPALL            2        /* Commande POPALL                    */


/*
    2    Structures and typedefs
    ____________________________
*/

/*
   2.1  Hardware dependant set of LED driver procedures - Led_desc
   ---------------------------------------------------------------
*/


typedef struct                         /* Adresses proceduresde bas niveau  */
  BEGIN                                /* ---------------------------------  */
    void      (*info)()              ; /* Recuperation des infos             */
    void      (*set )()              ; /* Definit l'etat de la LED           */
  END         Led_desc               ; /* ---------------------------------- */


/*
   2.2  LED controller context structure - Ledctrl_ctx
   ---------------------------------------------------
*/

#ifdef NDRV_LED_C

typedef struct                         /* Context for one LED's controller   */
  BEGIN                                /* ---------------------------------- */
    UINT      iodcar                 ; /* Controller VMOS IOD                */
    UCHAR     fled                   ; /* Idx du contexte de sa 1ere LED.    */
    UCHAR     nbleds                 ; /* Nombre de LEDs rattachees au ctrl  */
    UINT      reg[NBREGCTR]          ; /* Registres NRTYPE NRNAME NRNBLED    */
  END         Ledctrl_ctx            ; /* ---------------------------------- */
     
#else

typedef struct                         /* Context for one LED's controller   */
  BEGIN                                /* ---------------------------------- */
    Req_ctx   rctx                   ; /* IOD du controleur                  */
    UINT      iodcar                 ; /* Controller VMOS IOD                */
    UCHAR     fled                   ; /* Idx du contexte de sa 1ere LED.    */
    UCHAR     nbleds                 ; /* Nombre de LEDs rattachees au ctrl  */
    UINT      reg[NBREGCTR]          ; /* Registres NRTYPE NRNAME NRNBLED    */
  END         Ledctrl_ctx            ; /* ---------------------------------- */
     
#endif

/*
   2.3  One LED context structure - Led_ctx
   ----------------------------------------

   A Text pattern has a maximum size of SZPATT characters. The smallest item
   it may contain, in terms of character size, is "ON:0," so 5 characters.
   So a text pattern may include a maximum of SZPATT/5 items. The maximum
   size of a compiled item is 3 bytes, therefore a compiled pattern size
   cannot exceed L_RUBAN = (SZPATT / 5) * 3 bytes
*/

#define L_RUBAN    ((SZPATT/5)*3)      /* Maximum size of a compiled pattern */
#define T_PILE                  4      /* Max empilable par l'utilisateur    */
#define T_RCU                  32      /* Max RCU associables a une LED      */

#ifdef NDRV_LED_C

typedef struct                         /* Context for one LED                */
  BEGIN                                /* ---------------------------------- */
    UINT        iodsub               ; /* Sub-channel IOD for that LED       */
    UCHAR       dev                  ; /* Device number of the controller    */
    UCHAR       ssc                  ; /* Our sub-channel number             */
    UCHAR       iled                 ; /* Index in "led_ctx[]"               */
    UCHAR       myvl                 ; /* AUD_LED logical way number         */
    void        (*set)()             ; /* Low level procedure address        */
                                       /*                                    */
    UINT        reg[NBREGLED]        ; /* Registers for this LED             */

    UINT        pstk                 ; /* Stack pointer                      */
    UCHAR       press                ; /* 1 if between PRESS and RELEASE     */
    USHORT      iref                 ; /* Ref of program byte string         */
    USHORT      ioff                 ; /* Offset of next executed instruction*/
    INT         idto                 ; /* Current time-out identifier        */
                                       /*                                    */
    UINT        vlrcu                ; /* Bitfield of associated RCU's       */
    UINT        rtercu[T_RCU]        ; /* RCU->LED route identifiers         */
    UINT        cmd                  ; /* Last issued command                */
                                       /*                                    */
  END           Led_ctx              ; /* ---------------------------------- */
#else

typedef struct                         /* Context for one LED                */
  BEGIN                                /* ---------------------------------- */
    UINT        iodsub               ; /* Sub-channel IOD for that LED       */
    UCHAR       dev                  ; /* Device number of the controller    */
    UCHAR       ssc                  ; /* Our sub-channel number             */
    UCHAR       iled                 ; /* Index in "led_ctx[]"               */
    USHORT      myvl                 ; /* AUD_LED logical way number         */
    void        (*set)()             ; /* Low level procedure address        */
                                       /*                                    */
    UINT        reg[NBREGLED]        ; /* Registers for this LED             */
    UCHAR       pattern              ; /* Compiled code of  PATTERN          */
    UCHAR       rcupatt              ; /* Compiled code of  RCUPATT          */
                                       /*                                    */
    UCHAR       progs[T_PILE+1]      ; /* Stack of compiled patterns         */
    UCHAR       press                ; /* 1 if between PRESS and RELEASE     */
    UCHAR       iref                 ; /* Ref of program byte string         */
    USHORT      ioff                 ; /* Offset of next executed instruction*/
    INT         idto                 ; /* Current time-out identifier        */
                                       /*                                    */
    UINT        vlrcu                ; /* Bitfield of associated RCU's       */
    UINT        rtercu[T_RCU]        ; /* RCU->LED route identifiers         */
                                       /*                                    */
  END           Led_ctx              ; /* ---------------------------------- */

#endif

/*
   2.4  String byte store context - Ledbs_ctx
   ------------------------------------------
*/

typedef struct                         /* Context for one byte string store  */
  BEGIN                                /* ---------------------------------- */
    USHORT*     oid_addr             ; /* Start of OID's storage             */
    USHORT*     off_addr             ; /* Start of string offsets array      */
    UCHAR*      str_addr             ; /* Start of byte strings area         */
                                       /* ---------------------------------- */
    USHORT      nboid                ; /* Count of OID's                     */
    USHORT      nbstr                ; /* Count of offsets/strings           */
    INT         sz                   ; /* Size in byte of byte strings area  */
                                       /* ---------------------------------- */
    USHORT      qlen                 ; /* Cuttent MUTEX queue length         */
    INT         lg                   ; /* Currently used bytes               */
  END           Ledbs_ctx            ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of DRV_LED_H.                  */

/*
    5  RAM data to be inclueded in board_xyz.c
    ___________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

#define LED_NBTOT (  LED_NB0           /* LEDs for controller 0       */ \
                   + LED_NB1           /* LEDs for controller 1       */ \
                   + LED_NB2           /* LEDs for controller 2       */ \
                   + LED_NB3           /* LEDs for controller 3       */ )

#define LED_NBVL  (  2                 /* 2 Reserved LW (alloc,mutex) */ \
                   + LEDCTRL_NB        /* LW for controllers          */ \
                   + LED_NBTOT         /* LW for LED's                */ )

const USHORT led_nbiod = LEDCTRL_NB  ; /* Number of LW for controllers       */
const USHORT led_nbvl  = LED_NBVL    ; /* Total number of LW                 */
const USHORT led_nbsub = LED_NBMAX   ; /* Max count of LED for 1 controller  */

Ledctrl_ctx ledctrl_ctx [LEDCTRL_NB] ; /* Contexts for controllers           */
Led_ctx     led_ctx     [LED_NBTOT ] ; /* Contexts for LEDs                  */
USHORT      led_etat    [LED_NBVL  ] ; /* LW states                          */

Itf_ctx led_itfctx[1+LEDCTRL_NB]     ; /* 1 MODD and LEDCTLR_NB IODs         */
USHORT  led_sub_vl[LED_NBTOT]        ; /* Conversion ssc -> LW number        */

#endif                                 /* End of BOARD_XYZ_C                 */
