/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_rcu.h"             */
/*  Constants and definitions for RCU driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 02/05/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
            1.1  Receiver types
            1.2  Protocols codes
            1.3  Bits encoding types codes
            1.4  Special symbols identifiers
            1.5  Dedicated key codes

         2  Structures and typedefs
            2.1  Symbols description - Rcu_descsym and Rcu_sym
            2.2  RCU key description - Rcu_key
            2.3  RCU description - Rcu_def
            2.4  High level driver parameters - Rcu_drv_def
            2.5  Low-level interface procedures - Rcu_desc

         3  Public variable

         4  Prototypes of public procedures


*/


#ifndef DRV_RCU_H                      /* If this file is not already        */
#define DRV_RCU_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*  1.1  Receiver types
    -------------------
*/

#define RCU_NONE              0        /* Power off the receiver             */
#define RCU_IR                1        /* Select IR input                    */
#define RCU_UHF               2        /* Select UHF input                   */
#define RCU_PWM               3        /* Select baseband input              */
#define RCU_MAX               4        /* Count                              */


/*  1.2  Protocols codes
    --------------------
*/

#define PRO_RAW               0        /* Symbols raw capture                */
#define PRO_NEC               1        /* Tail codes                         */
#define PRO_RPT               2        /* Repeat messages                    */
#define PRO_RC6               3        /* RC6 mode 0                         */
#define PRO_RC5               4        /* RC5                                */
#define PRO_XMP               5        /* Universal Electronics XMP          */
#define PRO_CMM               6        /* RCMM                               */
#define PRO_HE                7        /* HomeEasy protocol (32 bits)        */
#define PRO_HEL               8        /* HomeEasy protocol (36 bits)        */
#define PRO_NB                9        /* Protocols count                    */


/*  1.3  Bit encoding type codes
    ----------------------------
*/

#define COD_OFFDUR            0        /* OFF duration encoding              */
#define COD_OFFMAN            1        /* OFF1-OFF2 durations encoding       */
#define COD_RC5               5        /* RC5 encoding                       */
#define COD_RC6               6        /* RC6 encoding                       */
#define COD_HE                7        /* HomeEasy protocol (32 bits)        */
#define COD_HEL               8        /* HomeEasy protocol (36 bits)        */


/*  1.4  Special symbols identifiers
    --------------------------------
*/
#define SYM_LEADER           -1        /* Leader      symbol identifier      */
#define SYM_DATFIRST         -2        /* First data  symbol identifier      */
#define SYM_DATLAST          -3        /* Last  data  symbol identifier      */
#define SYM_TAIL             -4        /* Tail code   symbol identifier      */
#define SYM_PSTOP            -5        /* Last  pulse symbol identifier      */
#define SYM_FPULSE           -6        /* First pulse symbol identifier      */
#define SYM_LASTID           -7        /* LAST IDENTIFIER value              */


/*  1.5  Dedicated key codes
    ------------------------
*/
#define KEY_1_OFF            KEY_SHIFT_F1  /* Device 1: OFF                  */
#define KEY_1_ON             KEY_SHIFT_F2  /* Device 1: ON                   */
#define KEY_2_OFF            KEY_SHIFT_F3  /* Device 2: OFF                  */
#define KEY_2_ON             KEY_SHIFT_F4  /* Device 2: ON                   */
#define KEY_3_OFF            KEY_SHIFT_F5  /* Device 3: OFF                  */
#define KEY_3_ON             KEY_SHIFT_F6  /* Device 3: ON                   */

#define KEY_GRP_OFF          KEY_SHIFT_F11 /* Group of devices: OFF          */
#define KEY_GRP_ON           KEY_SHIFT_F12 /* Group of devices: ON           */


/*
    2    Structures and typedefs
    ____________________________
*/

/*  2.1  Symbols description - Rcu_descsym and Rcu_sym
         ----------------------------------------------
*/

typedef struct                         /* Cette structure decrit les symboles*/
  BEGIN                                /* ---------------------------------- */
    UCHAR           nbsym            ; /* Nombre de symboles de la liste     */
    UCHAR           leader           ; /* Numero du symbole "leader"         */
    UCHAR           datfirst         ; /* Numero du 1er symbole de donnees   */
    UCHAR           datlast          ; /* Numero du dernier symbole donnees  */
    UCHAR           tail             ; /* Numero du symbole "tail-code"      */
    UCHAR           pstop            ; /* Numero du symbole pulse de fin     */
    UCHAR           bitpsym          ; /* Nombre de bits par symbole data    */
    UCHAR           fpulse           ; /* Numero du symbole qui definit pulse*/
  END               Rcu_descsym      ; /* ---------------------------------- */

typedef struct                         /* Cette structure decrit UN symbole  */
  BEGIN                                /* = bornes min et max de OT et ST    */
    USHORT          otmin            ; /* Valeur minimale de "on time"     us*/
    USHORT          otmax            ; /* Valeur minimale de "on time"     us*/
    USHORT          stmin            ; /* Valeur minimale de "symbol time" us*/
    USHORT          stmax            ; /* Valeur maximale de "symbol time" us*/
    USHORT          otnom            ; /* Valeur nominale de "on time"     us*/
    USHORT          stnom            ; /* Valeur nominale de "symbol time" us*/
  END               Rcu_sym          ; /* ---------------------------------- */


/*  2.2  RCU key description - Rcu_key
         -----------------------------
*/

typedef struct                         /* 1 touche d'1 telecommande          */
  BEGIN                                /* ---------------------------------- */
    USHORT          codercu          ; /* Code recu de la telecommande       */
    USHORT          codevmos         ; /* Code transmis au VMOS              */
  END               Rcu_key          ; /* ---------------------------------- */



/*  2.3  RCU description - Rcu_def
    ------------------------------
*/

typedef struct                         /* One RCU description                */
  BEGIN                                /* ---------------------------------- */
    CHAR           *name             ; /* RCU name                           */

/* Link level RCU description ...............................................*/

    UCHAR           link             ; /* RCU_IR/RCU_UHF/RCU_PWM             */
    UINT            freq             ; /* Carrier frequency in Hz            */
    UINT            banduhf          ; /* UHF frequency band in Hz (UHF only)*/
    UINT            power2           ; /* (Values in structure "Rcu_sym") *  */
                                       /* (2 ^ "power2") = times in microseco*/

/* Protocol level description ...............................................*/

    UCHAR           typro            ; /* RCU protocol                       */
    UCHAR           bitenc           ; /* Bits encoding method               */
    Rcu_descsym    *descsym          ; /* Global symbols description         */
    Rcu_sym        *sym              ; /* Symbols description list address   */
    UINT            toerr            ; /* TO attente apres erreur            */

    /* Donnees de description des touches "telecommande" ................... */

    UCHAR           rcu_nbsymdata    ; /* Nombre de symboles data du message */
    UCHAR           rcu_codeshft     ; /* Shift pour le code de touche       */
    UINT            rcu_codemsk  [2] ; /* Masque pour le code de touche      */
    UINT            rcu_idval    [2] ; /* Valeur du RCU ID                   */
    UINT            rcu_idmsk    [2] ; /* Masque pour le RCU Id              */
    UINT            rcu_relval   [2] ; /* Valeur de touche relachee          */
    UINT            rcu_relmsk   [2] ; /* Masque pour valeur touche relachee */
    Rcu_key        *rcu_key          ; /* Adresse liste des touches          */
    UINT            rcu_torep        ; /* TO repetition tail code/message    */

    /* Donnees de description des touches "clavier IR" ..................... */

    UCHAR           kbd_nbsymdata    ; /* Nombre de symboles data du message */
    UCHAR           kbd_codeshft     ; /* Shift pour le code de touche       */
    UINT            kbd_codemsk  [2] ; /* Masque pour le code de touche      */
    UINT            kbd_idval    [2] ; /* Valeur du RCU ID                   */
    UINT            kbd_idmsk    [2] ; /* Masque pour le RCU Id              */
    UINT            kbd_relval   [2] ; /* Valeur de touche relachee          */
    UINT            kbd_relmsk   [2] ; /* Masque pour vla touche relachee    */
    Rcu_key        *kbd_key          ; /* Adresse liste des touches          */
    UINT            kbd_torep        ; /* TO repetition tail code/message    */

    /* Donnees de description de la fonction "souris/boule" ................ */

    UCHAR           mse_nbsymdata    ; /* Nombre de symboles data du message */
    UCHAR           mse_codeshft     ; /* Shift pour le code de touche       */
    UINT            mse_codemsk  [2] ; /* Masque pour le code de touche      */
    UINT            mse_idval    [2] ; /* Valeur du RCU ID                   */
    UINT            mse_idmsk    [2] ; /* Masque pour le RCU Id              */
    Rcu_key        *mse_mov          ; /* Adresse liste des mouvements       */
    UINT            mse_torep        ; /* TO repetition message souris       */


  END               Rcu_def          ; /* ---------------------------------- */


/*  2.4  High level driver parameters - Rcu_drv_def
    -----------------------------------------------
*/

typedef struct                         /* Input High-level parameters        */
  BEGIN                                /* ---------------------------------- */
    UCHAR           link             ; /* RCU_IR/RCU_UHF/RCU_PWM             */
    UCHAR           signal           ; /* 0:normal  1:inverted               */
    CHAR           *defrcu           ; /* Default RCU name                   */
    UINT            freq             ; /* Carrier frequency                  */
    UINT            banduhf          ; /* Band frequency (UHF only)          */
  END               Rcu_modem        ; /* ---------------------------------- */


typedef struct                         /* Variable length list of elt's      */
  BEGIN                                /* ---------------------------------- */
    UCHAR           typdev           ; /* 0:Receiver  1:Transmitter          */
    UCHAR           defswpos         ; /* Default active input (0:first)     */
    UCHAR           nbswpos          ; /* Number of switch positions         */
    Rcu_modem       modem[]          ; /* Input's description                */
  END               Rcu_drv_def      ; /* ---------------------------------- */


/*  2.5  Low-level interface procedures
    -----------------------------------
*/

typedef struct                         /* Device RCU                         */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*init_rcurcv) () ; /* RCURCV initialization function     */
    void           (*end_rcurcv)  () ; /* RCURCV termination function        */
    void           (*mute_rcurcv) () ; /* Mute/Unmute a receiver             */
                                       /* ---------------------------------- */
    IWORD          (*init_rcusnd) () ; /* RCUSND initialization function     */
    void           (*end_rcusnd)  () ; /* RCUSND termination function        */
    void           (*start_rcusnd)() ; /* Starts sending a list of symbols   */
    void           (*abort_rcusnd)() ; /* Aborts sending a list of symbols   */
  END               Rcu_desc         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD  get_rcu_nb          (INT*,INT*,INT*,INT*                              );
IWORD  get_rcu_id          (INT,INT,INT*,INT*                                );
IWORD  get_switch_info     (INT,INT*,INT*,INT*                               );
IWORD  get_modem_info      (INT,INT,INT*,INT*,INT*,INT*,CHAR*,INT*           );
IWORD  get_rcu_info        (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,INT*,INT*     );
IWORD  get_rcu_sym         (INT,INT,INT*,INT*,INT*,INT*,INT*,INT*,INT*       );

IWORD  encode_simple       (                                                 );
IWORD  encode_nec          (                                                 );
IWORD  encode_rc6          (                                                 );
IWORD  encode_rc5          (                                                 );

IWORD  get_rcu             (INT,INT*,INT*,CHAR*,INT*                         );
IWORD  set_rcu             (INT,INT,const CHAR*,INT*                         );
IWORD  mute_rcu            (INT,INT,INT,INT*                                 );
IWORD  send_rcu            (INT,USHORT*,INT,INT,INT,INT*                     );
IWORD  continue_rcu        (INT,INT,INT*                                     );
IWORD  capture_rcu         (INT,INT,INT,INT,INT,INT*                         );
IWORD  rcu_ecrfa           (IWORD,IWORD,IWORD,UCHAR*,IWORD,UINT              );


#endif                                 /* End of DRV_RCU_H.                  */
