/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drvmio.h"              */
/*  Constants and definitions for I/O container drivers.                     */
/*                                                                           */
/*  Author : JYA                        Last modification : 07/23/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  Structures de description drivers systemes (ROM)
            2.2  Structures de description drivers/protocoles/services (ROM)
            2.3  Structure de description d'un board (ROM)
            2.4  Structure de description du driver systeme ecran (ROM)
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef DRVMIO_H                       /* If this file is not already        */
#define DRVMIO_H                       /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1 Structures de description drivers systemes (ROM)
   ----------------------------------------------------
*/

typedef struct                         /* VMIO system driver interface       */
  BEGIN                                /* ---------------------------------- */
    const CHAR      *drv_name        ; /* Driver name                        */
    UCHAR           typdrv           ; /* Always 0 with a system driver      */
                                       /* Interface procedures ------------- */
    IWORD           (*init)()        ; /* Initialization procedure           */
    IWORD           (*end )()        ; /* Termination procedure              */
  END               Sdrv_desc        ; /* ---------------------------------- */


typedef struct                         /* Interface d'un device              */
  BEGIN                                /* ---------------------------------- */
    const CHAR      *dev_name        ; /* Nom du device systeme (inutilise)  */
    const void      *dev_desc        ; /* Low-level procedure addresses      */
    const void      *dev_def         ; /* Low-level parameters address       */
    const void      *drv_def         ; /* High-level parameters address      */
  END               Dev_link         ; /* ---------------------------------- */
 

typedef struct                         /* Description d'un driver systeme    */
  BEGIN                                /* ---------------------------------- */
    const Sdrv_desc *sdrv_desc       ; /* Ad. interface  du  driver          */
    const Dev_link  *dev_link        ; /* Ad. interfaces des devices         */
    UCHAR           nb_dev           ; /* Nombre de devices dans sdev_link[] */
  END               Sdrv_link        ; /* ---------------------------------- */
 

/*
   2.2 Structures de description drivers/protocoles/services (ROM)
   ---------------------------------------------------------------
*/
 
typedef struct                         /* VMIO telecom/pipe interface        */
  BEGIN                                /* ---------------------------------- */
    IWORD           level            ; /* 1=driver, 2 et + = protocole       */
    CHAR            *name            ; /* layer name                         */
                                       /* Get Context/info addr ------------ */
    IWORD           (*get_rctx)   () ; /* Returns the Req_ctx context addr   */
    IWORD           (*get_info)   () ; /* Returns requested info data addr   */
                                       /* Interface procedure addr --------- */
    IWORD           (*connect)    () ; /* Outgoing connexion request         */
    IWORD           (*listen)     () ; /* Incoming connexion request         */
    IWORD           (*disconnect) () ; /* Negociated deconnexion request     */
    IWORD           (*abort)      () ; /* Abort connexion request            */
    IWORD           (*start)      () ; /* Start data flow request            */
    IWORD           (*stop)       () ; /* Stop data flow request             */
    IWORD           (*send)       () ; /* Data block send request            */
    IWORD           (*receive)    () ; /* Data block receive request         */
    IWORD           (*setval)     () ; /* Write registers request            */
    IWORD           (*getval)     () ; /* Read registers request             */
    IWORD           (*rpc)        () ; /* Execute command request            */
    IWORD           (*io)         () ; /* Composite I/O request              */

  END               Tlcm_desc        ; /* ---------------------------------- */

typedef struct                         /* VMIO Driver/proto/serv interface   */
  BEGIN                                /* ---------------------------------- */
    const CHAR      *drv_name        ; /* Driver name                        */
    UCHAR           typdrv           ; /* 001:dcar 010:proto 011:ser 101:dblk*/
                                       /* Initialization ------------------- */
    UCHAR           *initdone        ; /* Initialization flag address        */
    IWORD           (*init   )()     ; /* Specific initialization proc addr  */
                                       /* iniaut_s parameters -------------- */
    IWORD           aut              ; /* VMIO automaton number              */
    const S_trans   *trans           ; /* Transition table address           */
    const USHORT    *lim             ; /* States limits table address        */
    USHORT          *etat            ; /* State variables table address      */
                                       /* Const counts --------------------- */
    const USHORT    *nbvl            ; /* Size in USHORT's of "etat[]"       */
    const USHORT    *nbiod           ; /* Size in USHORTs of sessid/con2vl   */
    const USHORT    *nbmodd          ; /* Count of stacking contexts         */
    const USHORT    *msubcnt         ; /* Max Sub-channel count per device   */
                                       /* VMIO telecom/pipe interface ------ */
    const Tlcm_desc *tlcm            ; /* Tlcm_desc Interface procedures     */
    void            *adr             ; /* User address                       */
  END               Drv_desc         ; /* ---------------------------------- */


typedef struct _pipe_desc              /* Interface support pipe des drivers */
  BEGIN                                /* ---------------------------------- */

    /* Interface a implementer pour la source ...............................*/

    void          (*get_info_src)()  ; /*                                    */
    IWORD         (*init_src)()      ; /*                                    */
    void          (*end_src)()       ; /*                                    */
    IWORD         (*input)()         ; /*                                    */
    IWORD         (*seek)()          ; /*                                    */
    void          (*free_src_data)() ; /*                                    */

    /* Interface a implementer pour le puits ................................*/

    void          (*get_info_sink)() ; /*                                    */
    IWORD         (*init_sink)()     ; /*                                    */
    void          (*end_sink)()      ; /*                                    */
    IWORD         (*output)()        ; /*                                    */

    /* Interface commune ....................................................*/

    void          (*event)()         ; /*                                    */
  END             Pipe_desc          ; /* ---------------------------------- */


typedef struct                         /* Description d'un driver caractere  */
  BEGIN                                /* ---------------------------------- */
    const Drv_desc  *drv_desc        ; /* Ad. interface  du  driver          */
    const Dev_link  *dev_link        ; /* Ad. liste d'interfaces des devices */
    UCHAR           nb_dev           ; /* Nombre de devices dans dev_link[]  */
    const Pipe_desc *pipe            ; /* Ad. interface pipe                 */
  END               Drv_link         ; /* ---------------------------------- */
 
 
/*
   2.3 Structure de description d'un board (ROM)
   ---------------------------------------------
*/
 
typedef struct                         /* Description d'un board             */
  BEGIN                                /* ---------------------------------- */
    const CHAR      *board_name      ; /* Nom du board                       */
    const Drv_link  *const* drv_link ; /* Ad. tab Ad. de drivers/proto/serv  */
  END               Board_link       ; /* ---------------------------------- */
 

/*
    3    Public variables
    _____________________
*/

extern const Board_link *board          ; /* Board description               */

extern const Sdrv_desc sdrv_vmio        ; /* VMIO      system drv interface  */
extern const Sdrv_desc sdrv_ram         ; /* RAM       system drv interface  */

extern const Sdrv_desc sdrv_asy         ; /* ASY       system drv interface  */
extern const Sdrv_desc sdrv_dma         ; /* DMA       system drv interface  */
extern const Sdrv_desc sdrv_eep         ; /* EEP       system drv interface  */
extern const Sdrv_desc sdrv_gpio        ; /* GPIO      system drv interface  */
extern const Sdrv_desc sdrv_i2c         ; /* I2C       system drv interface  */
extern const Sdrv_desc sdrv_int         ; /* INT       system drv interface  */
extern const Sdrv_desc sdrv_kbd         ; /* KEYBOARD  system drv interface  */
extern const Sdrv_desc sdrv_mse         ; /* MOUSE     system drv interface  */
extern const Sdrv_desc sdrv_pcr         ; /* PCR       system drv interface  */
extern const Sdrv_desc sdrv_pipe        ; /* PIPE      system drv interface  */
extern const Sdrv_desc sdrv_rcu         ; /* RCU       system drv interface  */
extern const Sdrv_desc sdrv_rslv        ; /* RESOLVE   system drv interface  */
extern const Sdrv_desc sdrv_usb         ; /* USB       system drv interface  */

extern const Drv_desc drv_desc_asy      ; /* ASY       char drv interface    */
extern const Drv_desc drv_desc_gpio     ; /* GPIO      char drv interface    */
extern const Drv_desc drv_desc_usbitf   ; /* USB       char drv interface    */

extern const Drv_desc drv_desc_adc      ; /* ADC       char drv interface    */
extern const Drv_desc drv_desc_asi      ; /* ASI       char drv interface    */
extern const Drv_desc drv_desc_aud      ; /* AUD       char drv interface    */
extern const Drv_desc drv_desc_dac      ; /* DAC       char drv interface    */
extern const Drv_desc drv_desc_den      ; /* DEN       char drv interface    */
extern const Drv_desc drv_desc_dmx_small; /* DMX       char drv interface    */
extern const Drv_desc drv_desc_dmx_full ; /* DMX       char drv interface    */
extern const Drv_desc drv_desc_eth      ; /* ETH       char drv interface    */
extern const Drv_desc drv_desc_fre      ; /* FRE       char drv interface    */
extern const Drv_desc drv_desc_kbditf   ; /* KBDITF    char drv interface    */
extern const Drv_desc drv_desc_led      ; /* LED       char drv interface    */
extern const Drv_desc drv_desc_pow      ; /* POW       char drv interface    */
extern const Drv_desc drv_desc_sec      ; /* SEC       char drv interface    */
extern const Drv_desc drv_desc_smc      ; /* SMC       char drv interface    */
extern const Drv_desc drv_desc_swi      ; /* SWI       char drv interface    */
extern const Drv_desc drv_desc_vid      ; /* VID       char drv interface    */

extern const Drv_desc drv_desc_dsk_ata  ; /* DSK_ATA   block drv interface   */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of DRVMIO_H.                   */
