/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/memoire.h"             */
/*  Constants and definitions for memory manager.                            */
/*                                                                           */
/*  Author : JYA/RG                     Last modification : 10/03/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
            1.1  Constantes de dimensionnement de tables
            1.2  Tailles, en nombre d'octets
            1.3  Masque/shift pour les identifiants "memuid".

         2  Structures and typedefs
            2.1  Allocations headers - Alloc_head, Buf_head and Huge_head
            2.2  Contenu d'un buffer tetes de chainage - Buf_link
            2.3  Contexte de partition de RAM - Ram_def_seg et Mem_ctx
            2.4  Debug

         3  Public variable

         4  Prototypes of public procedures
            4.1  Partitions et identifiants utilisateurs
            4.2  Buffers
            4.3  Chainages de buffers
            4.4  Grosses allocations
            4.5  Listes
            4.6  Descriptionssss de classes
            4.7  Classes a acces direct
            4.8  Classes a acces sequentiel
            4.9  Procedures de bas niveau.
            4.10 Subroutine of mem_buf.c to be used by aut_membuf.c
            4.11 Subroutines of aut_membuf.c to be used by mem_buf.c

*/


#ifndef MEMOIRE_H                      /* If this file is not already        */
#define MEMOIRE_H                      /* included, include it.              */


/*
    1    Defines
    ____________

         Le present fichier include contient la definition des l'interfaces des
modules du repertoire "vmos/memoire", c'est-a-dire des modules "mem_buf.c",
"mem_list.c", "mem_class.c", "mem_object.c" et "mem_objstrm.c".


    1.1  Constantes de dimensionnement de tables
    --------------------------------------------

     MMAXPART est le nombre d'elements du tableau statique "mem_ctx[]", qui se
     trouve dans le fichier "mem_buf.c". C'est donc le nombre maximum de
     partitions memoires.

     MMAXSEG est le nombre d'elements du champ "seg[]" de la structure 
     "Mem_ctx". C'est donc le nombre maximum de segments de RAM d'une 
     partition memoire.
*/

#define MMAXPART   4                   /* Nombre maximum de partitions.      */
#define MMAXSEG    4                   /* Nbre max de segments par partition.*/


/*  1.2  Tailles, en nombre d'octets
    --------------------------------
*/

#define SBH             32             /* Egal a sizeof(Buf_head)            */
#define SAH             18             /* Egal a sizeof(Alloc_head)          */

#define ALIGN32_MIN_SZ 113             /* Taille min. pour disposer d'un     */
                                       /* buffer dont l'adresse de debut est */
                                       /* alignee sur 32 octets.             */



/*  1.3  Masque/shift pour les identifiants "memuid"
    ------------------------------------------------
*/

#define SHIFT_POOL         12          /*                                    */
#define MASK_POOL          0x3000      /*                                    */
                                       /* Positionnement des bits 12 et 13   */
                                       /* a la valeur de la partition (part) */
                                       /* selectionnee pour les alloc_bufs   */
                                       /* dans le memuid                     */
#define SET_MEMUID_POOL(memuid, pool) \
    (   ( (memuid) & ~MASK_POOL   )   \
      | ( ((pool) << SHIFT_POOL )     \
          & MASK_POOL             ) )

/*
    2    Structures et typedefs
    ___________________________


    2.1  Allocations headers - Alloc_head, Buf_head and Huge_head
    -------------------------------------------------------------

*/

typedef struct b_head                  /* En-tete buffer telecom             */
  BEGIN                                /* ---------------------------------- */
    struct b_head  *next             ; /* Chainage avant                   4 */
    struct b_head  *prev             ; /* Chainage arriere                 4 */
    UINT           pos               ; /* Position 1er octet utile         4 */
    UINT           nb                ; /* Nombre d'octets utiles           4 */
    UINT           taille            ; /* Taille en octets/code de taille  4 */
    UINT           bigof7            ; /* Offset dans le gros bloc         4 */
    USHORT         big               ; /* Numero du "gros bloc"            2 */
    USHORT         uid               ; /* Numero d'ordre de l'utilisateur  2 */
    USHORT         flags             ; /* Liste de flags                   2 */
    USHORT         ref               ; /* Compteur de references           2 */
  END              Buf_head          ; /* ------------------------------- 32 */
 

typedef struct                         /* En-tete allocation deplacable      */
  BEGIN                                /* ---------------------------------- */
    UINT           ind               ; /* Indice dans la table d'index     4 */
    UINT           taille            ; /* Taille utile en octets           4 */
    USHORT         nbulk             ; /* Nombre d'elements pour un bulk   2 */
    USHORT         verrou            ; /* Thread proprietaire/nb verrous   2 */
    USHORT         uid               ; /* Numero d'ordre de l'utilisateur  2 */
    USHORT         flags             ; /* Liste de flags                   2 */
    USHORT         ref               ; /* Compteur de references           2 */
  END              Alloc_head        ; /* ------------------------------- 18 */
 

typedef struct                         /* En-tete allocation "huge"          */
  BEGIN                                /* ---------------------------------- */
    UINT           pos               ; /* Position 1er octet utile         4 */
    UINT           nb                ; /* Nombre d'octets utiles           4 */
    USHORT         nbblk             ; /* Nombre de gros blocs consecutifs 2 */
    USHORT         uid               ; /* Numero d'ordre de l'utilisateur  2 */
    USHORT         flags             ; /* Liste de flags                   2 */
    USHORT         ref               ; /* Compteur de references           2 */
  END              Huge_head         ; /* ------------------------------- 16 */



 
/*  2.2  Contenu d'un buffer tetes de chainage - Buf_link
    -----------------------------------------------------

*/

typedef struct                         /* Structure buffer tete de chaine    */
  BEGIN                                /* ---------------------------------- */
    Buf_head       *next             ; /* Chainage avant                   4 */
    Buf_head       *prev             ; /* Chainage arriere                 4 */
    UINT           pos               ; /* Position 1er octet utile         4 */
    UINT           nb                ; /* Nombre d'octets utiles           4 */
    UINT           taille            ; /* Taille en octets/code de taille  4 */
    UINT           bigof7            ; /* Offset dans le gros bloc         4 */
    USHORT         big               ; /* Numero du "gros bloc"            2 */
    USHORT         uid               ; /* Numero d'ordre de l'utilisateur  2 */
    USHORT         flags             ; /* Liste de flags                   2 */
    USHORT         ref               ; /* Compteur de references           2 */

    UINT           maxelt            ; /* Nb max d'element chaines         4 */
    UINT           nbelt             ; /* Nb d'element chaines             4 */
    Buf_head       *last             ; /* Ptr sur dernier elt de la chaine 4 */
    UINT           rsz               ; /* Taille reele totale de la chaine 4 */
    UINT           usz               ; /* Taille utile totale de la chaine 4 */
    UINT           other[5]          ; /* Libres, 20 octets.              20 */
  END              Buf_link          ; /* ------------------------------- 72 */


#define LNKDATOF7  20                  /* Offset des donnees modifiables dans*/
                                       /* un buffer tete de chaine. Il s'agit*/
                                       /* de l'offset du champ "other" de la */
                                       /* Structure Buf_link SANS COMPTER LE */
#define LNKDATSZ   20                  /* Nb d'octets de donnees modifiables */
                                       /* dans une tete de chaine.           */


/*  2.3  Contexte de partition de RAM - Ram_def_seg et Mem_ctx
    ----------------------------------------------------------
*/


/* Structures decrivant la RAM du VMOS ......................................*/

typedef struct                         /* Descripteur d'un segment de RAM    */
  BEGIN                                /* ---------------------------------- */
    UCHAR          *adr              ; /* Adresse de debut du segment.       */
    UCHAR          *end              ; /* Adresse dernier octet segment.     */
    UINT           size              ; /* Taille du segment.                 */
    USHORT         flags             ; /* Liste de flags                     */
                                       /* b0  1:dans le cache  0:hors cache  */
  END              Ram_def_seg       ; /* ---------------------------------- */


typedef struct                         /* Les gros blocs d'un segment de RAM */
  BEGIN                                /* ---------------------------------- */
    INT            first             ; /* Numero du premier gros bloc        */
    INT            last              ; /* Numero du dernier gros bloc        */
    UINT           nbblk             ; /* Nombre de gros blocs               */
    UCHAR          *adr              ; /* Ad 1er octet du 1er gros bloc      */
    UCHAR          *end              ; /* Ad dernier octet dernier bloc, +1  */
  END              Blk_seg           ; /* ---------------------------------- */
 
#define NBSIZE      16                 /* Nb de tailles dans les tables.     */

typedef struct                         /* Description of a RAM partition     */
  BEGIN                                /* ---------------------------------- */
    UINT            nbseg            ; /* Number of segments (2 at least)    */
    Ram_def_seg     seg   [MMAXSEG]  ; /* Descriptions of the RAM segments   */
    Blk_seg         blkseg[MMAXSEG]  ; /* Big blocks of each segments        */
    Buf_head        *pall [NBSIZE ]  ; /* Head of pre-allocated buffer links */
                                       /*                                    */
    UINT            init             ; /* Is set to 1 at initialisation time */
    UINT            part             ; /* Number of this partition           */
    UINT            sbigb            ; /* Size of a big buffer               */
    UINT            dec              ; /* Memory split selected              */
    UINT            tmax             ; /* Rank of the maximum size           */
                                       /*                                    */
    UINT            indmax           ; /* Size of the index table            */
    UCHAR           **hindex         ; /* Ad of the index table              */
    UINT            bignb            ; /* Sizes of the two next tables       */
    UCHAR           **bigbuf         ; /* Ad of the big buffer address table */
    UINT            *bigctl          ; /* Ad of the 1st control area         */
    UINT            indfree          ; /* Head of chain of the free "hindex" */
                                       /*                                    */
    UINT            allocated        ; /* Debug, No of bytes allocated       */
    UINT            max_alloc        ; /* Debug, Max no of bytes allocated   */
    UINT            szsys            ; /* Debug, Size of the system zone     */
    UINT            chkblkbuf        ; /* Debug, for "chk_blkbuf"            */
  END               Mem_ctx          ; /* ---------------------------------- */


/* Structures pour le driver systeme RAM ....................................*/

typedef struct                         /* Procs optionnelles d'allocation    */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*alloc_ttx )()   ; /* Alloc Memoire teletexte (inutilise)*/
    IWORD          (*alloc_vmos)()   ; /* Alloc Memoire allocations VMOS     */
    IWORD          (*alloc_srv )()   ; /* Alloc Memoire services             */
    IWORD          (*alloc_dev )()   ; /* Alloc Memoire devices              */
    IWORD          (*alloc_part)()   ; /* Alloc Memoire partitions buffers   */
  END              Ram_desc          ; /* ---------------------------------- */


typedef struct                         /* Description stockage teletexte     */
  BEGIN                                /* ---------------------------------- */
    UCHAR           *adr             ; /* Adresse de stockage des pages      */
    USHORT          nbp              ; /* Nombre de pages stockables         */
  END               Ram_def_ttx      ; /* ---------------------------------- */


typedef struct                         /* Description memoire VMOS           */
  BEGIN                                /* ---------------------------------- */
    UCHAR           *adr             ; /* Adresse de la zone reservee au VMOS*/
    UINT            mem_sze          ; /* Taille de la zone reservee au VMOS */
    UINT            sml_sze          ; /* Taille reservee aux petites alloc. */
  END               Ram_def_vmos     ; /* ---------------------------------- */


typedef struct                         /* Description memoire Services       */
  BEGIN                                /* ---------------------------------- */
    UCHAR           *adr             ; /* Adresse de stockage des services   */
    UINT            sze              ; /* Taille de la zone de stockage.     */
  END               Ram_def_srv      ; /* ---------------------------------- */

typedef struct                         /* Description memoire devices        */
  BEGIN                                /* ---------------------------------- */
    UCHAR           *base            ; /* Adresse de base.                   */

    UINT            vid_bbf0_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf0_sze     ; /* video 0.                           */
    UINT            vid_bbf1_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf1_sze     ; /* video 1.                           */
    UINT            vid_bbf2_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf2_sze     ; /* video 2.                           */
    UINT            vid_bbf3_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf3_sze     ; /* video 3.                           */
    UINT            vid_bbf4_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf4_sze     ; /* video 4.                           */
    UINT            vid_bbf5_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf5_sze     ; /* video 5.                           */
    UINT            vid_bbf6_off     ; /* Offset et taille du bit-buffer     */
    UINT            vid_bbf6_sze     ; /* video 6.                           */
    UINT            num_vid_pvr      ; /* Numero du bit-buffer PVR.          */
    UINT            vid_buf_off      ; /* Offset et taille du buffer pour    */
    UINT            vid_buf_sze      ; /* les images decompressees.          */

    UINT            aud_bbf_off      ; /* Offsets et tailles des buffers     */
    UINT            aud_bbf_sze      ; /* video avant et apres decodage.     */
    UINT            aud_bbf_nb       ; /*                                    */
    UINT            aud_buf_off      ; /*                                    */
    UINT            aud_buf_sze      ; /*                                    */

    UINT            stl_buf_off      ; /* Offset et taille du buffer des     */
    UINT            stl_buf_sze      ; /* pixmaps des sous-titres DVB.       */
 
    UINT            osd_buf_off      ; /* Offset et taille de la zone de     */
    UINT            osd_buf_sze      ; /* stockage des buffers des OSD.      */

    UINT            aud_pvr_off      ; /* Offset et taille des buffers audio */
    UINT            aud_pvr_sze      ; /* pour le PVR                        */
    UINT            aud_pvr_nb       ; /*                                    */
    UINT            stl_pvr_off      ; /* Offset et taille du buffer STL pour*/
    UINT            stl_pvr_sze      ; /* le PVR.                            */

  END               Ram_def_dev      ; /* ---------------------------------- */
 
 
typedef struct                         /* Description memoire Telecom        */
  BEGIN                                /* ---------------------------------- */
    UINT            nbseg            ; /* Nombre de segments dans partition  */
    INT             sbigb            ; /* Taille d'un bigbuffer.             */
    Ram_def_seg     seg[4]           ; /* Descripteurs des segments.         */
  END               Ram_def_part     ; /* ---------------------------------- */
 

typedef struct                         /* Description d'un mapping memoire   */
  BEGIN                                /* ---------------------------------- */
    const Ram_def_ttx   *ttx         ; /* Memoire pages teletexte            */
    const Ram_def_vmos  *vmos        ; /* Memoire allocations VMOS           */
    const Ram_def_srv   *srv         ; /* Memoire services                   */
    const Ram_def_dev   *dev         ; /* Memoire devices                    */
    const Ram_def_part  *part        ; /* Memoire partitions buffers         */
    UINT                nbpart       ; /* Nb de descriptions de partitions   */
  END                   Ram_def      ; /* ---------------------------------- */


/*  2.4  Debug
    ----------
*/

typedef struct                         /* One buffer log table entry         */
  BEGIN                                /* ---------------------------------- */
    CHAR                *proc        ; /* Procedure name                     */
    UCHAR               *buf         ; /* Buffer address                     */
    INT                 sz           ; /* Requested allocation size          */
  END                   Buf_log      ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/*  4.1  Partitions et identifiants utilisateurs ............................*/

  IWORD init_mem            (INT,INT,INT*                                    );
  IWORD end_mem             (INT,INT*                                        );
  IWORD alloc_memuid        (INT*,INT*                                       );
  IWORD free_memuid         (INT,INT,INT*                                    );
  IWORD get_part_dbg        (INT,INT*,INT*,INT*,INT*                         );
  IWORD get_part_info       (INT,INT*,INT*,INT*,INT*                         );
  IWORD test_allocadr       (UCHAR*,INT*,INT*,INT*                           );
  IWORD check_part          (INT,INT*,INT*,UCHAR**,UCHAR**,INT*              );
  IWORD check_user_str      (IWORD                                           );


/*  4.2  Buffers ............................................................*/

  IWORD alloc_buf           (INT,INT,INT,UCHAR**,INT*                        );
  IWORD realloc_buf         (INT,UCHAR**,INT,INT,INT*                        );
  IWORD post_buf            (UCHAR*,INT*                                     );
  IWORD free_buf            (UCHAR*,INT*                                     );
  IWORD alloc_aligned_buf   (INT,INT,INT,INT,INT,INT,INT,UCHAR**,INT*,INT*   );
  IWORD get_buf_size        (UCHAR*,INT*,INT*,INT*,INT*                      );
  IWORD get_buf_part        (UCHAR*,INT*,INT*                                );
  IWORD set_buf_posnb       (UCHAR*,INT,INT,INT*                             );
  IWORD prealloc_buf        (INT,INT,INT,INT,INT*                            );
                            

/*  4.3  Chainages de buffers ...............................................*/

  IWORD alloc_link          (INT,INT,INT,UCHAR**,INT*                        );
  IWORD post_link           (UCHAR*,INT*                                     );
  IWORD free_link           (UCHAR*,INT*                                     );
  IWORD link_buf            (UCHAR*,UCHAR*,INT,INT,INT,INT*                  );
  IWORD relink_buf          (UCHAR*,UCHAR*,UCHAR*,INT,INT,INT,INT*           );
  IWORD unlink_buf          (UCHAR*,UCHAR*,INT*                              );
  IWORD next_buf            (UCHAR*,UCHAR**,INT*,INT*,INT*                   );
  IWORD search_bufval       (UCHAR*,INT,INT,INT,INT,UCHAR**,INT*             );
  IWORD free_all_buf        (UCHAR*,INT*                                     );
  IWORD get_link_size       (UCHAR*,INT*,INT*,INT*,INT*                      );
                            
                            
/*  4.4  Grosses allocations ................................................*/
                            
  IWORD alloc_huge          (INT,INT,INT,UCHAR**,INT*                        );
  IWORD realloc_huge        (INT,UCHAR**,INT,INT,INT*                        );
  IWORD post_huge           (UCHAR*,INT*                                     );
  IWORD free_huge           (UCHAR*,INT*                                     );
  IWORD get_huge_size       (UCHAR*,INT*,INT*,INT*                           );
  IWORD get_huge_part       (UCHAR*,INT*,INT*                                );


/*  4.5  Listes .............................................................*/


/*  4.6  Descriptions de classes ............................................*/

  IWORD init_class          (void                                            );
  IWORD new_class           (INT,CHAR*,INT,INT,INT*,INT*                     );
  IWORD add_class_var       (INT,CHAR*,INT,INT,INT,INT,INT,USHORT[]           ,
                             INT,INT*,INT*                                   );
  IWORD add_class_subclass  (INT,CHAR*,INT,INT*,INT*,INT*                    );
  IWORD add_class_sto       (INT,INT,INT,INT,INT*                            );
  IWORD enable_class        (INT,INT*                                        );
  IWORD free_class          (INT,INT*                                        );
  IWORD get_class_id        (INT,CHAR*,INT*,INT*                             );
  IWORD get_class_name      (INT,INT*,CHAR**,INT*                            );
  IWORD get_class_desc      (INT,CHAR**,INT*,INT*,INT*,INT*                  );
  IWORD get_var_id          (INT,CHAR*,INT*,INT*                             );
  IWORD get_var_fullname    (INT,INT*,CHAR*,INT*                             );
  IWORD get_var_tree        (INT,CHAR*,INT*,INT*,INT*,INT*                   );
  IWORD get_var_info        (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,USHORT[]      ,
                             INT*,INT*                                       );
  IWORD get_sto_info        (INT,INT,INT*,INT*,INT*,INT*                     );


/*  4.7  Classes a acces direct .............................................*/

  IWORD set_dirclass_value  (UCHAR*,INT,UCHAR*,INT,INT*                      );
  IWORD get_dirclass_value  (UCHAR*,INT,UCHAR*,INT,INT*                      );


/*  4.8  Classes a acces sequentiel .........................................*/

  IWORD alloc_seqctx        (INT,INT*,INT*                                   );
  IWORD set_seqctx_adr      (INT,UCHAR*,INT,INT*                             );
  IWORD get_seqctx_adr      (INT,INT,UCHAR**,INT*,INT*                       );
  IWORD set_seqctx_break    (INT,INT,INT,INT,INT*,INT*                       );
  IWORD get_seqctx_break    (INT,INT,INT*,INT*                               );
  IWORD parse_seqctx        (INT,INT*,INT*                                   );
  IWORD next_seqctx         (INT,INT*,INT*                                   );
  IWORD get_seqctx_value    (INT,INT*,UCHAR*,INT,INT*                        );
  IWORD free_seqctx         (INT,INT*                                        );


/*  4.9  Procedures de bas niveau ...........................................*/

  void  flush_dcache        (UCHAR*,INT                                      );
  void  invalidate_dcache   (UCHAR*,INT                                      );
  void  exclude_dcache      (INT,UCHAR*,INT                                  );


/*  4.10 Subroutine of mem_buf.c to be used by aut_membuf.c .................*/

  IWORD alloc_bl            (Mem_ctx*,UINT,UINT,UCHAR**                      );
  void  init_buflink        (UCHAR*,INT                                      );

/*  4.11 Subroutines of aut_membuf.c to be used by mem_buf.c ................*/

  void  init_aut_membuf     (void                                            );
  IWORD alloc_snd_evt       (Mem_ctx*,UINT,UINT,USHORT,USHORT,INT            );
  IWORD realloc_snd_evt     (INT,UCHAR*,INT,INT                              );
  IWORD free_snd_evt        (UCHAR*,USHORT                                   );


#endif                                 /* End of MEMOIRE_H.                  */
