/*
**  Myapp_sem.c - Sample code for HyperpanelOS ============================  **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  At start-up we spawn two tasks: a "producer" which sends a custom event  **
**  every 1 second; and a "consumer" which chooses to be locked on a         **
**  semaphore.                                                               **
**  =======================================================================  **
*/


/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces
#include <drv_asy.h>                   // Prototype of "asy_write"

/* Internal data types of the module ........................................*/

static int     producer_proc    (void *                                     ) ;
static int     consumer_proc    (void *                                     ) ;
static int     task_free        (void                                       ) ;


/* Internal defines of this module ------------------------------------------*/

#define TICK         10000             // Code for tick event
#define MSG_MAX      80                // Message max size


/* Internal global variables of this module ---------------------------------*/

int            producer_stack[512]   ; // The stack of our task (2kB)
int            producer_taskid       ; // Our task id

int            consumer_stack[512]   ; // The stack of our task (2kB)
int            consumer_taskid       ; // Our task id

int            nb_tasks              ; // Number of tasks
int            sem_id                ; // Semaphore id
int            nb_semop              ; // Number of displayed messages


/* Beginning of the code ------------------------------------------------------

init_app_tsk        VMK Task creation function

producer_proc       Entry point for create_task - Producer main loop
consumer_proc       Entry point for create_task - Consumer main loop
task_free           Termination function for create_task

*/



/*  Procedure init_app_tsk ----------------------------------------------------

    Purpose : Allocates task contexts and associated Logical Ways of
              the AUT_TASK automaton, then requests the start of the task.
-----------------------------------------------------------------------------*/

int init_app_tsk(Task_grp *g,char *mod,char*conf)
  {
    int             ret              ; // Returned code

/*****************************************************************************
 * Step 1 : The "create_task" procedure initializes all the data that is     *
 * ------   necessary for a new task, it allocates a task context and a      *
 *          logical way of the AUT_TASK automaton. This LW will be used to   *
 *          receive all the events for that task. The returned identifier    *
 *          "taskid" is in fact equal to 0x12FFFF00 | LW, where LW is a      *
 *          Logical way number of AUT_TASK, and also an index in the context *
 *          table "task_ctx[]". The input parameters of "create_task" are    *
 *          stored into "task_ctx[LW]":                                      *
 *          - The task entry point is always a C procedure and here our      *
 *            procedures are "consumer_proc" and "producer_proc".            *
 *          - In the event that the task entry point procedure returns, the  *
 *            VMK will call a "clean-up" procedure we have to provide. Here  *
 *            our termination procedure is "task_free".                      *
 *          - The stack addresses ("consumer_stack" and "producer_stack")    *
 *            and their sizes in bytes.                                      *
 *          - The task Prioritys and the task initialization flags.          *
 *****************************************************************************/

    create_task("apptsk"              ,  // Task group name
                consumer_proc         ,  // Task entry point
                task_free             ,  // Task termination call-back
                consumer_stack        ,  // Stack address
                sizeof(consumer_stack),  // Stack size in bytes
                PRIO_TSK_MIN          ,  // Task priority
                HNULL                 ,  // Parameter for "mytask_proc"
                TASK_INIT_BSS  +         // Init flags
                TASK_INIT_DATA +         //
                TASK_INIT_CODE        ,  //
                &consumer_taskid      ); // Task_id = 0x12FFFF00 + LW

    create_task("apptsk"              , // Task group name
                producer_proc         , // Task entry point
                task_free             , // Task termination call-back
                producer_stack        , // Stack address
                sizeof(producer_stack), // Stack size in bytes
                PRIO_TSK_MIN          , // Task priority
                HNULL                 , // Parameter for "mytask_proc"
                TASK_INIT_BSS  +        // Init flags
                TASK_INIT_DATA +        //
                TASK_INIT_CODE       ,  //
                &producer_taskid     ); // Task_id = 0x12FFFF00 + LW

    nb_tasks = 2                      ; // Number of tasks


/*****************************************************************************
 * Step 2 : Here the new tasks have been created but this does not implies   *
 * ------   they start running. A call to "start_task" has to be issued.     *
 *          This procedure sends an event (RUN_TASK) that is a request to    *
 *          start task execution. This event is sent to the                  *
 *          "*_taskid & 0xFF" logical way of automaton AUT_TASK.             *
 *          ---------------------------------------------------------------- *
 *          Here we only start "consummer". The "producer" is in this        *
 *          sample code started by the "consumer"                            *
 *****************************************************************************/

    start_task(consumer_taskid,        // Send an event to VMK in order
               &ret            )     ; // to request the task start


/****************************************************************************
 * Step 3 : Initialize the driver/protocol/services interface. Once done,   *
 * -------  the VMK "app" can use the driver/protocole/services interface   *
 *          through procedures named "drv_***". Those procedures are        *
 *          sending REQ_xyz events to the VMIO automatons.                  *
 ****************************************************************************/

    init_telecom()                   ; // Initialize driver/protocol/services
                                       // API

/*****************************************************************************
 * Step 4 : Create one semaphore and store its identifier in the "sem_id"    *
 * ------   Global variable                                                  *
 *****************************************************************************/

    create_semaphore(0, &sem_id)     ; // Init semaphore (1 resource)

    return 0                         ; // Exit without any error
  }


/*  Procedure consumer_proc ---------------------------------------------------

    Purpose : This is the consumer task.
*/

static INT consumer_proc(void *param)
  {
    char            msg[MSG_MAX]     ; // Message buffer to be sent
    int             ret              ; // Returned code

/*****************************************************************************
 * Step 1 : Here we start the "producer", using a call to "start_task"       *
 * ------   This procedure sends an event (RUN_TASK) that is a request to    *
 *          start task execution. This event is sent to the                  *
 *          "*_taskid & 0xFF" logical way of automaton AUT_TASK.             *
 *****************************************************************************/

    start_task(producer_taskid,        // Send an event to VMK in order
               &ret            )     ; // to request the task start


/*****************************************************************************
 * Step 2 : In an infinite loop:                                             *
 * ------   - We wait for the semaphore. We are unscheduled until the        *
 *            producer executes a "put_semaphore" on "sem_id".               *
 *          - When we are scheduled again, we send a message                 *
 *****************************************************************************/

    loop                             : // Infinite loop begin label

    get_semaphore(sem_id, &ret)      ; // We are unscheduled until the
                                       // producer calls "put_semaphore"

    hsprintf(msg            ,          // Here we are scheduled again. We
             "get %d %d\r\n",          // build a message in
             ret, nb_semop     )     ; // "msg"

    asy_write(0                ,       // Use ASY0
           (unsigned char*)msg ,       // Message to be sent
           Strlen(msg)          )    ; // Count of bytes to be sent

    goto loop                        ; // New loop step

    return 0                         ; // Exit without any error
  }


/*  Procedure producer_proc ---------------------------------------------------

    Purpose : This is the producer task.
*/

static INT producer_proc(void *param)
  {
    char            msg[MSG_MAX]     ; // Message to be sent on ASY0
    int             ret              ; // Returned code

/*****************************************************************************
 * Step 1 : In an infinite loop:                                             *
 * ------   - We wait for 1 second.                                          *
 *          - We call "put_semaphore", semaphore counter goes from 0 to 1    *
 *          - We send a message on the serial line                           *
 *          - Then we loop again and we ask to be unscheduled for one second *
 *            The consequence is that the consumer becomes scheduled again,  *
 *            its call to "get_semaphore" completes after the semaphore      *
 *            counter has been decremented (1 -> 0). The consumer send a     *
 *            message and calls "get_semaphore" again, its is unscheduled    *
 *            because the semaphore counter is now 0.                        *
 *****************************************************************************/

    loop                              : // Infinite loop begin label

    waitevt_task(HNULL    ,            // Address of waiting list = none
                 0        ,            // Size of "waitlist[]"
                 1000     ,            // maximum waiting time = 1000 millisec
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code

    put_semaphore(sem_id, &ret)      ; // Increment the semaphore counter

    nb_semop ++                      ; // Here we are scheduled again. We
                                       // increment the counter of put/get

    hsprintf(msg            ,          // We build a message that we send on
             "put %d %d\r\n",          // serial line ASY0
             ret, nb_semop     )     ; // 

    asy_write(0                ,       // Use ASY0
           (unsigned char*)msg ,       // Message to be sent
           Strlen(msg)           )   ; // Count of bytes to be sent

    goto loop                        ; // New loop step

    return 0                         ; // Exit without any error
  }


/*  Procedure task_free -------------------------------------------------------

    Purpose : This is our task "clean-up" procedure for both tasks.
*/

static int task_free(void)
  {

/*****************************************************************************
 * Step 1 : We decrement the number of running tasks. If last task of this   *
 * ------   application is dead, we continue with step 2.                    *
 *****************************************************************************/

    if (--nb_tasks)                    // If a task is still running,
      goto end                       ; // we skip step 2.

/*****************************************************************************
 * Step 2 : We send a message on serial line ASY0                            *
 * ------                                                                    *
 *****************************************************************************/

    asy_write(0 ,
         (unsigned char*)"end\r\n",
          5                        ) ;

    end                              : // Early exit.

    return 0                         ; // Exit without any error
  }

