/*
**  Myapp_tasks.c - Sample code for HyperpanelOS ==========================  **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  At start-up we spawn two tasks: a "producer" which sends a custom event  **
**  every 500 milliseconds and a "consumer" which receives the event and     **
**  prints a message on serial line ASY0.                                    **
**  =======================================================================  **
*/

/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces
#include <drv_asy.h>                   // Prototype of "asy_write"

/* Internal data types of the module ........................................*/

static int     producer_proc    (void*                                      ) ;
static int     consumer_proc    (void*                                      ) ;
static int     task_free        (void                                       ) ;

/* Internal defines of this module ------------------------------------------*/

#define  START                  10000  // Code for start event
#define  POKE                   10001  // Code for poke event
                                

/* Internal global variables of this module ---------------------------------*/

int            producer_stack[512]   ; // The stack of our task (512B)
int            producer_taskid       ; // Our task id

int            consumer_stack[512]   ; // The stack of our task (512B)
int            consumer_taskid       ; // Our task id

int            nb_tasks              ; // Number of tasks
int            poke_id  = 0          ; // Identifier of poke



/* Beginning of the code ----------------------------------------------------

init_app_tsk        VMK Task creation function

producer_proc       Entry point for create_task - Producer main loop
consumer_proc       Entry point for create_task - Consumer main loop
task_free           Termination function for create_task

*/


/*  Procedure init_app_tsk ----------------------------------------------------

    Purpose : Allocates task contexts and associated Logical Ways of
              the AUT_TASK automaton, then requests the start of the tasks.
*/

int init_app_tsk(Task_grp *g,char *mod,char*conf)
  {
    int             ret              ; // Returned code

/*****************************************************************************
 * Step 1 : The "create_task" procedure initializes all the data that is     *
 * ------   necessary for a new task, it allocates a task context and a      *
 *          logical way of the AUT_TASK automaton. This LW will be used  to  *
 *          receive all the events for that task. The returned identifier    *
 *          "taskid" is in fact equal to 0x12FFFF00 | LW, where LW is a      *
 *          Logical way number of AUT_TASK, and also an index in the context *
 *          table "task_ctx[]". The input parameters of "create_task" are    *
 *          stored into "task_ctx[LW]" :                                     *
 *          - The task entry point is always a C procedure and here our      *
 *            procedures are "consumer_proc" and "producer_proc".            *
 *          - In the event that the task entry point procedure returns, the  *
 *            VMK will call a "clean-up" procedure we have to provide. Here  *
 *            our termination procedure is "task_free".                      *
 *          - The stack addresses ("consumer_stack" and "producer_stack")    *
 *            and their sizes in bytes.                                      *
 *          - The task Prioritys and the task initialization flags           *
 *****************************************************************************/

    create_task("apptsk"        ,      // Taskgroup name
                consumer_proc   ,      // Task entry point
                task_free       ,      // Tasm termination call-back
                consumer_stack  ,      // Stack address
                sizeof(consumer_stack),// Stack size in bytes
                PRIO_TSK_MIN    ,      // Task Priority
                HNULL           ,      // Parameter for "mytask_proc"
                TASK_INIT_BSS  +       // Init Flags
                TASK_INIT_DATA +       //
                TASK_INIT_CODE  ,      //
                &consumer_taskid  )  ; // Task_id = 0x12FFFF00 + LC

    create_task("apptsk"        ,      // Taskgroup name
                producer_proc   ,      // Task entry point
                task_free       ,      // Tasm termination call-back
                producer_stack  ,      // Stack address
                sizeof(producer_stack),// Stack size in bytes
                PRIO_TSK_MIN    ,      // Task Priority
                HNULL           ,      // Parameter for "mytask_proc"
                TASK_INIT_BSS  +       // Init Flags
                TASK_INIT_DATA +       //
                TASK_INIT_CODE  ,      //
                &producer_taskid  )  ; // Task_id = 0x12FFFF00 + LC

    nb_tasks = 2                     ; // Number of tasks


/*****************************************************************************
 * Step 2 : Here the new task has been created but this does not implies it  *
 * ------   starts running. A call to "start_task" has to be issued. This    *
 *          procedure sends an event (RUN_TASK) that is a request to start   *
 *          task execution. This event is sent to the "*_taskid & 0xFF"      *
 *          logical way of automaton AUT_TASK.                               *
 *****************************************************************************/


    start_task(consumer_taskid,        // Send an event to VMK in order
               &ret            )     ; // to request the task start

    start_task(producer_taskid,        // Send an event to VMK in order
               &ret            )     ; // to request the task start

/****************************************************************************
 * Step 3 : Initialize the driver/protocol/services interface. Once done,   *
 * -------  the VMK "app" can use the driver/protocole/services interface   *
 *          through procedures named "drv_***". Those procedures are        *
 *          sending REQ_xyz events to the VMIO automatons.                  *
 ****************************************************************************/

    init_telecom()                   ; // Initialize driver/protocol/services
                                       // API

    return 0                         ; // Exit without any error
  }


/*  Procedure consumer_proc ---------------------------------------------------

    Purpose : This is consumer task main loop.
*/

static INT consumer_proc(void *param)
  {
    int             waitlist[1][3]   ; // Parameter of "waitevt_task"
    int             d, m, y          ; // Day, Month, Year
    int             h, mi, s, ms     ; // Hour, minutes, seconds, milliseconds
    int             ret              ; // Returned code
    char            msg[80]          ; // Message sent on serial line ASY0

/*****************************************************************************
 * Step 2 : Route a START event. As we have one (and only one) route that    *
 * ------   "catches" this  event (the one created by the producer") this    *
 *          call generates one and only one copy of the START event, that    *
 *          is received by the produces as soon as we will unschedule        *
 *****************************************************************************/

    sendevt_task(producer_taskid ,     // Target task identifier
                 START           ,     // code
                 0               ,     // reserve
                 HNULL           ,     // adresse
                 0               ,     // longueur
                 0               ,     // flags
                 0               ,     // res2
                 0               ,     // reqid
                 &ret              ) ; // Error code


/*****************************************************************************
 * Step 3 : This is our task event loop. We wait for the "producer_proc"     *
 * ------   generated event, that has POKE as code. We call "wait_coderes"   *
 *          that unschedules us until POKE is received. Then we read current *
 *          date and time, build a one line message that we send on the      *
 *          "asy_iod0" serial line. Then we loop.                            *
 *          ---------------------------------------------------------------- *
 *          As soon as we are unscheduled, the produces gets scheduled       *
 *          because it receives the START event we sent just above           *
 *****************************************************************************/


    waitlist[0][0]  = WAIT_CODERES   ; // Test "code" and "reserve" fields
    waitlist[0][1]  = POKE           ; // The awaited value for "code"
    waitlist[0][2]  = -1             ; // The awaited value for "reserve"

    loop :                             // OUr main loop label

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // No maximum waiting time
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code

    tim_get(&d  ,                      // Day
            &m  ,                      // Month
            &y  ,                      // Year
            &h  ,                      // Hour (0..23)
            &mi ,                      // Minutes (0..59)
            &s  ,                      // Seconds (0..59)
            &ms   )                  ; // Milliseconds (0..999)

    hsprintf(msg                  ,    // Buffer address
       "%02d:%02d:%02d <%08X>\r\n",    // HH:MM:SS <xxxxxxxx>
        h,   mi,  s               ,    //
        task_evt.reqid             ) ; //

    asy_write(0            ,
       (unsigned char*)msg ,
       Strlen(msg)           )       ;

    goto loop                        ;

    return 0                         ;
  }


/*  Procedure producer_proc ---------------------------------------------------

    Purpose : This is producer task main loop.
*/

static INT producer_proc(void *param)
  {

    int             ret              ; // Returned code
    int             waitlist[1][3]   ; // Parameter of "waitevt_task

/*****************************************************************************
 * Step 2 : Wait for START event from consumer and delete route when done.   *
 * ------                                                                    *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODERES   ; // Test "code" and "reserve" fields
    waitlist[0][1]  = START          ; // The awaited value for "code"
    waitlist[0][2]  = -1             ; // The awaited value for "reserve"

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // No maximum waiting time
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code



/*****************************************************************************
 * Step 3 : Send a POKE event to the consumer every two seconds. We route    *
 * ------   the POKE event and then we wait for a duration of 500 msec. To  *
 *          do this, we use the "waitevt_task" function with an empty list   *
 *          of awaited events and a maximum waiting duration of 500 msec     *
 *          The "reqid" event field value is the "poke_id" global variable   *
 *          that we increment at each loop step                              *
 *****************************************************************************/

    loop :                             // Our main loop label

    waitevt_task(HNULL    ,            // Waiting list address = none
                 0        ,            // Size of "waitlist[]"
                 500      ,            // Wait for 500 msec
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code

    sendevt_task(consumer_taskid ,     // Target task identifier
                 POKE            ,     // code
                 0               ,     // reserve
                 HNULL           ,     // adresse
                 0               ,     // longueur
                 0               ,     // flags
                 0               ,     // res2
                 poke_id ++      ,     // reqid
                 &ret              ) ; // Return code

    goto loop                        ; // Repeat


    return 0                         ;
  }

/*  Procedure task_free -------------------------------------------------------

    Purpose : This is our task "clean-up" procedure for both tasks.
*/

static int task_free(void)
  {

/*****************************************************************************
 * Step 1 : We decrement number of running tasks. If last task of this app   *
 * ------   is dead, we continue with step 2.                                *
 *****************************************************************************/

    if (--nb_tasks)                    // If a task is still running,
      goto end                       ; // we skip step 2.

/*****************************************************************************
 * Step 2 : We send a message on serial line ASY0                            *
 * ------                                                                    *
 *****************************************************************************/

    asy_write(0 ,
         (unsigned char*)"end\r\n",
          5                        ) ;

    end                              : // Early exit.

    return 0                         ;
  }

