/*
**  Mytsk.c - Sample code for simple APP task start ============ 09/05/19 =  **
**                                                                           **
**                                                                           **
**  =======================================================================  **
*/

/* Includes files and external references ...................................*/

#include <stdtyp.l>
#include <moteur.h>                    /* vmk.c and vmio.c interface         */
#include <drv.h>                       /* drv.c interface                    */

#include <memoire.h>                   /* mem_buf.c interface                */

#include <vmkm.txt>                    /* TSK_INIT_DATA TASK_INIT_BSS        */
#include <systemm.txt>                 /* WAIT_CODERES WAIT_CODEINT          */

extern INT loop_app_tsk(void *)      ;
static int end_app_tsk(void)         ;

/* Internal global variables for the user task ------------------------------*/

int                 mystack[512]     ; // The stack of our task (0.5kB)
int                 myapp_taskid     ; // Our task id
int                 myapp_memuid     ; // Our memory user id


/* Beginning of the code ----------------------------------------------------

------------------  User task creation and start ----------------------------
init_app_tsk        VMK user Task creation function
end_app_tsk         Termination function for create_task
wait_coderes        Wait for a specific (code,res) event
wait_allevent       Wait for the next event

*/

/*  Procedure init_app_tsk-----------------------------------------------------

    Purpose : Allocates a task context and an associated Logical Way of
              the AUT_TASK automaton, then requests the start of the task.
              This procedure is executed within VMK hardware context.
              The created task will be executed in a different hardware
              context, using as a stack the "mystack" array (taskid > 0).
*/

int init_app_tsk(Task_grp *g,char *mod, char*conf)
  {
    int             ret              ; // Returned code

/*****************************************************************************
 * Step 1 : The "create_task" procedure initializes all the data that is     *
 * ------   necessary for a new task, it allocates a task context and a      *
 *          logical way of the AUT_TASK automaton. This LW will be used to   *
 *          receive all the events for that task. The returned identifier    *
 *          "taskid" is in fact equal to 0x12FFFF00 | LW, where LW is a      *
 *          Logical Way number of AUT_TASK, and also an index in the context *
 *          table "task_ctx[]". The input parameters of "create_task" are    *
 *          stored into "task_ctx[LW]":                                      *
 *          - The task entry point is always a C procedure and here our      *
 *            procedure is "loop_app_tsk.                                    *
 *          - In the event that the task entry point procedure returns, the  *
 *            VMK will call a "clean-up" procedure we have to provide. Here  *
 *            our termination procedure is "myfree_proc".                    *
 *          - The stack address ("mystack") and its size in bytes.           *
 *          - The task Priority and the task initialization flags.           *
 *****************************************************************************/

    create_task("apptsk"        ,      // Taskgroup name
                loop_app_tsk    ,      // Task entry point
                end_app_tsk     ,      // Task termination call-back
                mystack         ,      // Stack address
                sizeof(mystack) ,      // Stack size in bytes
                PRIO_TSK_MIN    ,      // Task Priority
                HNULL           ,      // Parameter for "loop_app_tsk
                TASK_INIT_BSS  +       // Init Flags
                TASK_INIT_DATA +       //
                TASK_INIT_CODE  ,      //
                &myapp_taskid     )  ; // Task_id = 0x12FFFF00 + LW


/*****************************************************************************
 * Step 2 : Here the new task has been created but this does not imply it    *
 * ------   starts running. A call to "start_task" has to be issued. This    *
 *          procedure sends an event (RUN_TASK) that is a request to start   *
 *          task execution. This event is sent to the "myapp_task_id & 0xFF" *
 *          logical way of automaton AUT_TASK.                               *
 *****************************************************************************/

    start_task(myapp_taskid,           // Send an event to VMK in order
               &ret          )       ; // to request the task start


/*****************************************************************************
 * Step 3 : In order to be able to use the "alloc_buf" procedure one needs   *
 * ------   a user identifier from the allocation subsystem. We just have    *
 *          to make a call to "alloc_memuid", we get the identifier in the   *
 *          "myapp_memuid" global variable.                                  *
 *****************************************************************************/

    alloc_memuid(&myapp_memuid,        // Allocates a user id for alloc_buf
                 &ret          )     ; // and other allocation procedures


/****************************************************************************
 * Step 4 : Initialize the driver/protocol/services interface. Once done,   *
 * -------  the VMK "app" can use the driver/protocole/services interface   *
 *          through procedures named "drv_***". Those procedures are        *
 *          sending REQ_xyz events to the VMIO automatons.                  *
 ****************************************************************************/

    init_telecom()                   ; // Initialize driver/protocol/services
                                       // API

    return 0                         ; // Exit without any error
  }


/*  Procedure end_app_tsk -----------------------------------------------------

    Purpose : This is our task "clean-up" procedure.
*/

static int end_app_tsk(void)
  {
    int             ret              ; // Procedurs return code

/*****************************************************************************
 * Step 1 : We free the memory user identifier that was allocated by the     *
 * ------   "init_app_tsk" procedure.                                        *
 *****************************************************************************/

    free_memuid(myapp_memuid,          // Frees a user id for alloc_buf
                0x00000007  ,          // mask = buffers/links/huge
                &ret          )      ; //
    return 0                         ;
  }


/*  Procedure wait_coderes ----------------------------------------------------

    Purpose : Unschedule until one specific event is received. If non awaited
              events are received meanwhile, they will be kept inside the task
              events waiting queue.
*/

void wait_coderes(int code, int res)
  {
    int             ret              ; // Return code
    int             waitlist[1][3]   ; // Parameter of "waitevt_task

/*****************************************************************************
 * Step 1 : We build a list of a single pair (code,reserve). The task will   *
 * ------   be unschedule until an event corresponding to the one expected   *
 *          is received by the task. This wait may be infinite if the event  *
 *          is never received.                                               *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODERES   ; // Test "code" and "reserve" fields
    waitlist[0][1]  = code           ; // The awaited value for "code"
    waitlist[0][2]  = res            ; // The awaited value for "reserve"

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // No maximum waiting time
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code
  }


/*  Procedure wait_allevent ---------------------------------------------------

    Purpose : Unschedule until the next event is received, whatever it is.
*/

int wait_allevent(void)
  {
    int             waitlist[1][3]   ; // Parameter of "waitevt_task"
    int             ret              ; // Return code for "waiyevt_task"

/*****************************************************************************
 * Step 1 : Build a list with one WAIT_CODEINT entry that will accept all    *
 * ------   the event codes ranging from 0 to 20000. Then call               *
 *          "waitevt_task", we will be unscheduled until the next event will *
 *          be received                                                      *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODEINT   ; // All events with
    waitlist[0][1]  = 0              ; // a code between 0
    waitlist[0][2]  = 20000          ; // and 20000

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // maximum waiting time = no
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code


/*****************************************************************************
 * Step 2 : Here we are scheduled again. The VMK has written into its        *
 * ------   global variable "task_evt" a copy of the event that has          *
 *          scheduled us again.                                              *
 *****************************************************************************/

    return task_evt.code             ;

  }




