/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS - I/O container       File "include/space_iocont.c"        */
/*  IO container configurable static data                                    */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/11/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================

         1  Purpose of this module
            1.1  Definition of static RAM arrays
            1.2  Definition of static ROM (const) arrays

         2  Include files

         3  External imported variables

         4  Packages declaration.
            4.1 Package PKG_vmio
            4.2 Package PKG_req_lib
            4.3 Package PKG_vmit
            4.4 Package PKG_drvgen


    1    Purpose of this module
    ___________________________

    1.1  Definition of static RAM arrays

         This module contains all the static RAM arrays of the IO container. 
Each array size is a "define" which value is defined in one "space_xx.h" file.
By using different copies of the "space_xx.h" file we can customize the size of
the VMIO RAM arrays.


         The "space_xx.h" file is normally automatically generated using the
"create" tool from the Hyperpanel workshop. But for IOT's versions of the OS,
we do not deliver the workshop and in such a case the "space_xx.h" file is
created by the developer. We use the term "package" that corresponds to a fixed
set of C modules, for example the VMIO "PKG_vmio" package is made of one C module that
is "vmio.c". When this "package" (the "vmio.c" module) is used, we need to 
define several C static RAM arrays that are "fa_s[]", "fah_s[]", "tabto_s[]", 
"route_s[]", "tablim_s[]", "varetat_s[]" for the "VMIO". Also, the VMIO uses 
some "debug" tables that are "tdura_fa_s[]", tdura_trt_s[]". The definition of 
those arrays are located in the present "space_iocont.c" as follows:

#ifdef PKG_vmio                        /+ If package is used                 +/

/+ Other constants ..........................................................+/

#define        AUTS_LAST (AUTS_FIRST+MAXAUT_S-1)

#define        FAL       MAXAUT_S      /+ Index in "fah_s[]" for slow   evts +/
#define        FAN       MAXAUT_S + 1  /+ Index in "fah_s[]" for normal evts +/
#define        FAU       MAXAUT_S + 2  /+ Index in "fah_s[]" for urgent evts +/
#define        FFREE     MAXAUT_S + 3  /+ Index in "fah_s[]" for free evts   +/

/+ Tables for "vmio.c" ......................................................+/

S_trans        *tabtrans_s[MAXAUT_S] ; /+ Addresses of transition tables     +/
USHORT         *tablim_s  [MAXAUT_S] ; /+ Addr of transition table indexes   +/
USHORT         *varetat_s [MAXAUT_S] ; /+ Addresses of arrays of state var   +/
S_evt          fa_s       [MAXEVT_S] ; /+ Storage for all waiting queues     +/
Fah_s          fah_s    [MAXAUT_S+4] ; /+ Heads of waiting queus             +/
S_to_s         tabto_s    [NTOMAX_S] ; /+ Time-out table                     +/
Route          route_s    [MAXRTE_S] ; /+ Route table                        +/

/+ Table sizes as const variables ...........................................+/

const INT      maxaut_s  = MAXAUT_S  ; /+ Max number of FSM's                +/
const INT      maxevt_s  = MAXEVT_S  ; /+ Internal waiting queue size        +/
const INT      ntomax_s  = NTOMAX_S  ; /+ Max number of time-out's           +/
const INT      maxrte_s  = MAXRTE_S  ; /+ Max number of event's routes       +/

/+ Ofenly used array indexes ................................................+/

const INT      auts_last = AUTS_LAST ; /+ Last VMIO FSM identifier           +/

const INT      fal_idx = FAL         ; /+ Indice de la FAL   = maxaut_s + 0  +/
const INT      fan_idx = FAN         ; /+ Indice de la FAN   = maxaut_s + 1  +/
const INT      fau_idx = FAU         ; /+ Indice de la FAU   = maxaut_s + 2  +/
const INT      faf_idx = FFREE       ; /+ Indice de la FFREE = maxaut_s + 3  +/

/+ Times measurements for debug .............................................+/

Dura_fa_s      tdura_fa_s[MAXAUT_S+3][MAXDUR_S] ; /+ Duration in wait queue  +/
Dura_trt_s     tdura_trt_s[MAXAUT_S][MAXDUR_S]  ; /+ C treatments            +/
Dura_hand_it   tdura_hand_it[MAXDUR_IT]         ; /+ Interrupt handlers      +/
Dura_crit_it   tdura_crit_it[MAXDUR_IT]         ; /+ Critical sections       +/

const INT      tdura_fa_s_sze     = sizeof(tdura_fa_s)   ;
const INT      tdura_trt_s_sze   = sizeof(tdura_trt_s)   ;
const INT      tdura_hand_it_sze = sizeof(tdura_hand_it) ;
const INT      tdura_crit_it_sze = sizeof(tdura_crit_it) ;

#endif                                 /+ #ifdef PKG_VMIO                    +/


         In order to be compiled, this piece of code needs that the PKG_vmio
macro is defined, and that MAXAUT_S, MAXEVT_S, NTOMAX_S and MAXRTE_S are defined
as positive integer constants. Those will be located within one "space_xx.h"
file, that indeed is included in the present "space_ocont.c". Here below is an 
example of what will be found in "space_xx.h" :

#define PKG_vmio                       /+ Use the PKG_vmio package           +/

#define MAXAUT_S             16        /+ Maximum number of FSM's            +/
#define MAXEVT_S             16        /+ Internal waiting queue size        +/
#define NTOMAX_S              8        /+ Maximum number of Time-out's       +/
#define MAXRTE_S              8        /+ Maximum number of event's routes   +/


         It has to be noted that the source code of vmio.c is not provided to 
customers, that however must be able to customize the different sizes, without 
compiling again the source codes that are not provided. So the source code of
vmio.c the defines, but instead the static const variables "maxaut_s", 
"maxevt_s", "ntomax_s" and "maxrte_s". We also have other const variables
"fal_idx/fan_idx/fau_idx/faf_idx" that all are only used values of indexes in
the "fah_s[]" table.


    1.2  Definition of static ROM (const) arrays

         We may also use the same "package" mechanism to initialize const
arrays with values that are given as "defines" in "space_xx.h". For example,
the VMIT needs 8 stacks for the 8 levels of interrupts. We have two arrays of 8
entries that respectively contains the start and end addresses of the 8 stack.
Using constants from "space_xx.h", we can customize the stacks addresses and
sizes.


    2  Include files
    ________________

*/

#include <moteur.h>                    /* S_evt S_trans S_aut S_to Route     */
#include <automaton.h>                 /* Automaton numbers                  */
#include <vmiodat.h>                   /* Fah_s                              */
#include <vmiodbg.h>                   /*                                    */
#include <drvmio.h>                    /* req_lib.h needs Tlcm_desc          */
#include <req_lib.h>                   /* Job_data                           */


/*
    3    External imported variables
    ________________________________
*/

/* Depuis le linker ........................................................ */

extern IWORD     itstkl              ; /* Ad. basse des piles d'IT           */
extern IWORD     itstkh              ; /* Ad. haute des piles d'IT           */


/*
    4    Packages declaration
    _________________________
*/

/* 4.1 Package PKG_vmio -----------------------------------------------------*/

#ifdef PKG_vmio                        /* If package is used                 */

/* Other constants ..........................................................*/

#define        AUTS_LAST (AUTS_FIRST+MAXAUT_S-1)

#define        FAL       MAXAUT_S      /* Index in "fah_s[]" for slow   evts */
#define        FAN       MAXAUT_S + 1  /* Index in "fah_s[]" for normal evts */
#define        FAU       MAXAUT_S + 2  /* Index in "fah_s[]" for urgent evts */
#define        FFREE     MAXAUT_S + 3  /* Index in "fah_s[]" for free evts   */

/* Tables for "vmio.c" ......................................................*/

S_trans        *tabtrans_s[MAXAUT_S] ; /* Addresses of transition tables     */
USHORT         *tablim_s  [MAXAUT_S] ; /* Addr of transition table indexes   */
USHORT         *varetat_s [MAXAUT_S] ; /* Addresses of arrays of state var   */
S_evt          fa_s       [MAXEVT_S] ; /* Storage for all waiting queues     */
Fah_s          fah_s    [MAXAUT_S+4] ; /* Heads of waiting queus             */
S_to_s         tabto_s    [NTOMAX_S] ; /* Time-out table                     */
Route          route_s    [MAXRTE_S] ; /* Route table                        */

/* Table sizes as const variables ...........................................*/

const INT      maxaut_s  = MAXAUT_S  ; /* Max number of FSM's                */
const INT      maxevt_s  = MAXEVT_S  ; /* Internal waiting queue size        */
const INT      ntomax_s  = NTOMAX_S  ; /* Max number of time-out's           */
const INT      maxrte_s  = MAXRTE_S  ; /* Max number of event's routes       */

/* Ofenly used array indexes ................................................*/

const INT      auts_last = AUTS_LAST ; /* Last VMIO FSM identifier           */

const INT      fal_idx = FAL         ; /* Indice de la FAL   = maxaut_s + 0  */
const INT      fan_idx = FAN         ; /* Indice de la FAN   = maxaut_s + 1  */
const INT      fau_idx = FAU         ; /* Indice de la FAU   = maxaut_s + 2  */
const INT      faf_idx = FFREE       ; /* Indice de la FFREE = maxaut_s + 3  */

/* Times measurements for debug .............................................*/

Dura_fa_s      tdura_fa_s[MAXAUT_S+3][MAXDUR_S] ; /* Duration in wait queue  */
Dura_trt_s     tdura_trt_s[MAXAUT_S][MAXDUR_S]  ; /* C treatments            */
Dura_hand_it   tdura_hand_it[MAXDUR_IT]         ; /* Interrupt handlers      */
Dura_crit_it   tdura_crit_it[MAXDUR_IT]         ; /* Critical sections       */

const INT      tdura_fa_s_sze     = sizeof(tdura_fa_s)   ;
const INT      tdura_trt_s_sze   = sizeof(tdura_trt_s)   ;
const INT      tdura_hand_it_sze = sizeof(tdura_hand_it) ;
const INT      tdura_crit_it_sze = sizeof(tdura_crit_it) ;

#endif                                 /* #ifdef PKG_VMIO                    */


/* 4.2 Package PKG_req_lib --------------------------------------------------*/

#ifdef PKG_req_lib                     /* If Package is used                 */

Job_data       staticjob[NBJOB]      ; /* Pool of static Job_data structs    */
const INT      nbjob     = NBJOB     ; /* Size of the "staticjob[]" array    */

#endif                                 /* #ifdef PKG_req_lib                 */


/* 4.3 Package PKG_vmit -----------------------------------------------------*/

#ifdef PKG_vmit                        /* If Package is used                 */

/* Table of interrupt stack start addresses ................................ */

const IWORD *itstk[8] =                /* Table des adresses des piles d'IT  */
  BEGIN                                /* ---------------------------------- */
    &itstkl + OF0/ZW                ,  /* Pile pour IT de priorite 0         */
    &itstkl + OF1/ZW                ,  /* Pile pour IT de priorite 1         */
    &itstkl + OF2/ZW                ,  /* Pile pour IT de priorite 2         */
    &itstkl + OF3/ZW                ,  /* Pile pour IT de priorite 3         */
    &itstkl + OF4/ZW                ,  /* Pile pour IT de priorite 4         */
    &itstkl + OF5/ZW                ,  /* Pile pour IT de priorite 5         */
    &itstkl + OF6/ZW                ,  /* Pile pour IT de priorite 6         */
    &itstkl + OF7/ZW                ,  /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */

/* Table of interrupt stack end addresses .................................. */

const IWORD *endstk[8] =               /* Table des adresses fin de piles    */
  BEGIN                                /* ---------------------------------- */
    &itstkl + OF0/ZW - SZ0/ZW       ,  /* Pile pour IT de priorite 0         */
    &itstkl + OF1/ZW - SZ1/ZW       ,  /* Pile pour IT de priorite 1         */
    &itstkl + OF2/ZW - SZ2/ZW       ,  /* Pile pour IT de priorite 2         */
    &itstkl + OF3/ZW - SZ3/ZW       ,  /* Pile pour IT de priorite 3         */
    &itstkl + OF4/ZW - SZ4/ZW       ,  /* Pile pour IT de priorite 4         */
    &itstkl + OF5/ZW - SZ5/ZW       ,  /* Pile pour IT de priorite 5         */
    &itstkl + OF6/ZW - SZ6/ZW       ,  /* Pile pour IT de priorite 6         */
    &itstkl + OF7/ZW - SZ7/ZW       ,  /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */

#endif                                 /* #ifdef PKG_vmit                    */


/* 4.4 Package PKG_drvgen ---------------------------------------------------*/

#ifdef PKG_drvgen                      /* If Package is used                 */


#endif                                 /* #ifdef PKG_vmit                    */
